@@ -29,6 +29,12 @@ buffer* buffer_init(void) {
 	return b;
 }
 
+buffer *buffer_init_buffer(buffer *src) {
+	buffer *b = buffer_init();
+	buffer_copy_string_buffer(b, src);
+	return b;
+}
+
 /**
  * free the buffer 
  * 
@@ -504,33 +510,15 @@ int buffer_is_empty(buffer *b) {
 /**
  * check if two buffer contain the same data
  * 
- * this is a optimized 32/64bit compare function.
- * 
- * it is assumed that the leftmost byte have the most equality. 
- * That why the comparision is done right to left
- * 
+ * HISTORY: this function was pretty much optimized, but didn't handled
+ * alignment properly.
  */
 
 int buffer_is_equal(buffer *a, buffer *b) {
-	size_t i;
-	
 	if (a->used != b->used) return 0;
 	if (a->used == 0) return 1;
 
-	/* we are unsigned, if i < 0 it will flip to MAX_SIZE_T and will be > a->used */
-	for (i = a->used - 1; i < a->used && i % (sizeof(size_t)); i--) {
-		if (a->ptr[i] != b->ptr[i]) return 0;
-	}
-	
-	/* compare the single char itself which was kicked us out of the loop */ 
-	if (i < a->used && a->ptr[i] != b->ptr[i]) return 0;
-	
-	for (i -= (sizeof(size_t)); i < a->used; i -= (sizeof(size_t))) {
-		if (*((size_t *)(a->ptr + i)) != 
-		    *((size_t *)(b->ptr + i))) return 0;
-	}
-	
-	return 1;
+	return (0 == strcmp(a->ptr, b->ptr));
 }
 
 int buffer_is_equal_string(buffer *a, const char *s, size_t b_len) {
@@ -654,7 +642,7 @@ int buffer_append_string_hex(buffer *b, const char *in, size_t in_len) {
 	return 0;
 }
 
-int buffer_append_string_url_encoded(buffer *b, const char *s) {
+int buffer_append_string_url_encoded(buffer *b, const char *s, size_t s_len) {
 	unsigned char *ds, *d;
 	size_t d_len;
 	
@@ -664,6 +652,7 @@ int buffer_append_string_url_encoded(buffer *b, const char *s) {
 		SEGFAULT();
 	}
 	
+	if (s_len == 0) return 0;
 	
 	/* count to-be-encoded-characters */
 	for (ds = (unsigned char *)s, d_len = 0; *ds; ds++) {
@@ -741,9 +730,9 @@ int buffer_append_string_url_encoded(buffer *b, const char *s) {
 	return 0;
 }
 
-int buffer_append_string_html_encoded(buffer *b, const char *s) {
+int buffer_append_string_html_encoded(buffer *b, const char *s, size_t s_len) {
 	unsigned char *ds, *d;
-	size_t d_len;
+	size_t d_len, i;
 	
 	if (!s || !b) return -1;
 	
@@ -751,19 +740,25 @@ int buffer_append_string_html_encoded(buffer *b, const char *s) {
 		SEGFAULT();
 	}
 	
+	/* nothing to append */
+	if (s_len == 0) return 0;
 	
 	/* count to-be-encoded-characters */
-	for (ds = (unsigned char *)s, d_len = 0; *ds; ds++) {
+	for (ds = (unsigned char *)s, d_len = 0, i = 0; i < s_len && *ds; ds++, i++) {
 		d_len++;
-		if (*ds == '<' || *ds == '>')
+		if (*ds == '<' || *ds == '>') {
 			d_len += 4 - 1;
-		else if (*ds == '&')
+		} else if (*ds == '&') {
 			d_len += 5 - 1;
+		}
 	}
 
 	buffer_prepare_append(b, d_len);
 	
-	for (ds = (unsigned char *)s, d = (unsigned char *)b->ptr + b->used - 1, d_len = 0; *ds; ds++) {
+	for (ds = (unsigned char *)s, 
+	     d = (unsigned char *)b->ptr + b->used - 1, 
+	     d_len = 0,
+	     i = 0; i < s_len && *ds; ds++, i++) {
 		switch (*ds) {
 		case '>':
 			d[d_len++] = '&';
@@ -958,3 +953,32 @@ int light_isalpha(int c) {
 int light_isalnum(int c) {
 	return light_isdigit(c) || light_isalpha(c);
 }
+
+int buffer_to_lower(buffer *b) {
+	char *c;
+	
+	if (b->used == 0) return 0;
+	
+	for (c = b->ptr; *c; c++) {
+		if (*c >= 'A' && *c <= 'Z') {
+			*c |= 32;
+		}
+	}
+	
+	return 0;
+}
+
+
+int buffer_to_upper(buffer *b) {
+	char *c;
+	
+	if (b->used == 0) return 0;
+	
+	for (c = b->ptr; *c; c++) {
+		if (*c >= 'a' && *c <= 'z') {
+			*c &= ~32;
+		}
+	}
+	
+	return 0;
+}
