@@ -6,6 +6,7 @@
 
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <sys/mman.h>
 
 #include <stdlib.h>
 #include <fcntl.h>
@@ -35,8 +36,10 @@ static chunk *chunk_init(void) {
 	
 	c = calloc(1, sizeof(*c));
 	
-	/* c->mem overlaps with c->data.file.name */
-	c->data.mem = buffer_init();
+	c->mem = buffer_init();
+	c->file.name = buffer_init();
+	c->file.fd = -1;
+	c->file.mmap.start = MAP_FAILED;
 	c->next = NULL;
 	
 	return c;
@@ -45,16 +48,29 @@ static chunk *chunk_init(void) {
 static void chunk_free(chunk *c) {
 	if (!c) return;
 	
-	/* c->data.mem overlaps with c->data.file.name */
-	switch (c->type) {
-	case MEM_CHUNK: buffer_free(c->data.mem); break;
-	case FILE_CHUNK: buffer_free(c->data.file.name); break;
-	default: break;	
-	}
-	
+	buffer_free(c->mem);
+	buffer_free(c->file.name);
+
 	free(c);
 }
 
+static void chunk_reset(chunk *c) {
+	if (!c) return;
+	
+	buffer_reset(c->mem);
+	buffer_reset(c->file.name);
+
+	if (c->file.fd != -1) {
+		close(c->file.fd);
+		c->file.fd = -1;
+	}
+	if (MAP_FAILED != c->file.mmap.start) {
+		munmap(c->file.mmap.start, c->file.mmap.length);
+		c->file.mmap.start = MAP_FAILED;
+	}
+}
+
+
 void chunkqueue_free(chunkqueue *cq) {
 	chunk *c, *pc;
 	
@@ -86,6 +102,7 @@ static chunk *chunkqueue_get_unused_chunk(chunkqueue *cq) {
 		c = cq->unused;
 		cq->unused = c->next;
 		c->next = NULL;
+		cq->unused_chunks--;
 	}
 	
 	return c;
@@ -116,15 +133,26 @@ static int chunkqueue_append_chunk(chunkqueue *cq, chunk *c) {
 }
 
 void chunkqueue_reset(chunkqueue *cq) {
+	chunk *c;
 	/* move everything to the unused queue */
 	
-	if (cq->last == NULL) return;
-	
-	cq->last->next = cq->unused;
-	cq->unused = cq->first;
-	
-	/* disconnect active chain */
-	cq->first = cq->last = NULL;
+	/* mark all read written */ 
+	for (c = cq->first; c; c = c->next) {
+		switch(c->type) {
+		case MEM_CHUNK:
+			c->offset = c->mem->used - 1;
+			break;
+		case FILE_CHUNK:
+			c->offset = c->file.length;
+			break;
+		default: 
+			break;
+		}
+	}
+
+	chunkqueue_remove_finished_chunks(cq);
+	cq->bytes_in = 0;
+	cq->bytes_out = 0;
 }
 
 int chunkqueue_append_file(chunkqueue *cq, buffer *fn, off_t offset, off_t len) {
@@ -136,9 +164,9 @@ int chunkqueue_append_file(chunkqueue *cq, buffer *fn, off_t offset, off_t len)
 	
 	c->type = FILE_CHUNK;
 	
-	buffer_copy_string_buffer(c->data.file.name, fn);
-	c->data.file.offset = offset;
-	c->data.file.length = len;
+	buffer_copy_string_buffer(c->file.name, fn);
+	c->file.offset = offset;
+	c->file.length = len;
 	c->offset = 0;
 	
 	chunkqueue_append_chunk(cq, c);
@@ -154,7 +182,7 @@ int chunkqueue_append_buffer(chunkqueue *cq, buffer *mem) {
 	c = chunkqueue_get_unused_chunk(cq);
 	c->type = MEM_CHUNK;
 	c->offset = 0;
-	buffer_copy_string_buffer(c->data.mem, mem);
+	buffer_copy_string_buffer(c->mem, mem);
 	
 	chunkqueue_append_chunk(cq, c);
 	
@@ -169,7 +197,7 @@ int chunkqueue_prepend_buffer(chunkqueue *cq, buffer *mem) {
 	c = chunkqueue_get_unused_chunk(cq);
 	c->type = MEM_CHUNK;
 	c->offset = 0;
-	buffer_copy_string_buffer(c->data.mem, mem);
+	buffer_copy_string_buffer(c->mem, mem);
 	
 	chunkqueue_prepend_chunk(cq, c);
 	
@@ -184,7 +212,7 @@ int chunkqueue_append_mem(chunkqueue *cq, const char * mem, size_t len) {
 	c = chunkqueue_get_unused_chunk(cq);
 	c->type = MEM_CHUNK;
 	c->offset = 0;
-	buffer_copy_string_len(c->data.mem, mem, len - 1);
+	buffer_copy_string_len(c->mem, mem, len - 1);
 	
 	chunkqueue_append_chunk(cq, c);
 	
@@ -198,11 +226,11 @@ buffer * chunkqueue_get_prepend_buffer(chunkqueue *cq) {
 	
 	c->type = MEM_CHUNK;
 	c->offset = 0;
-	buffer_reset(c->data.mem);
+	buffer_reset(c->mem);
 	
 	chunkqueue_prepend_chunk(cq, c);
 	
-	return c->data.mem;
+	return c->mem;
 }
 
 buffer *chunkqueue_get_append_buffer(chunkqueue *cq) {
@@ -212,11 +240,11 @@ buffer *chunkqueue_get_append_buffer(chunkqueue *cq) {
 	
 	c->type = MEM_CHUNK;
 	c->offset = 0;
-	buffer_reset(c->data.mem);
+	buffer_reset(c->mem);
 	
 	chunkqueue_append_chunk(cq, c);
 	
-	return c->data.mem;
+	return c->mem;
 }
 
 off_t chunkqueue_length(chunkqueue *cq) {
@@ -226,10 +254,10 @@ off_t chunkqueue_length(chunkqueue *cq) {
 	for (c = cq->first; c; c = c->next) {
 		switch (c->type) {
 		case MEM_CHUNK:
-			len += c->data.mem->used ? c->data.mem->used - 1 : 0;
+			len += c->mem->used ? c->mem->used - 1 : 0;
 			break;
 		case FILE_CHUNK:
-			len += c->data.file.length;
+			len += c->file.length;
 			break;
 		default:
 			break;
@@ -261,52 +289,39 @@ int chunkqueue_is_empty(chunkqueue *cq) {
 	return cq->first ? 0 : 1;
 }
 
-#ifdef DEBUG_CHUNK
+int chunkqueue_remove_finished_chunks(chunkqueue *cq) {
+	chunk *c;
 
-static int write_chunkqueue(int fd, chunkqueue *c) {
-	UNUSED(fd);
-	UNUSED(c);
+	for (c = cq->first; c; c = cq->first) {
+		int is_finished = 0;
+
+		switch (c->type) {
+		case MEM_CHUNK:
+			if (c->offset == (off_t)c->mem->used - 1) is_finished = 1;
+			break;
+		case FILE_CHUNK:
+			if (c->offset == c->file.length) is_finished = 1; 
+			break;
+		default: 
+			break;
+		}
+
+		if (!is_finished) break;
+
+		chunk_reset(c);
+
+		cq->first = c->next;
+		if (c == cq->last) cq->last = NULL;
+
+		/* keep at max 4 chunks in the 'unused'-cache */
+		if (cq->unused_chunks > 4) {
+			chunk_free(c);
+		} else {
+			c->next = cq->unused;
+			cq->unused = c;
+			cq->unused_chunks++;
+		}
+	}
 
 	return 0;
 }
-
-int main(int argc, char **argv) {
-	chunkqueue *c;
-	buffer *b, *fn;
-	
-	UNUSED(argc);
-	UNUSED(argv);
-
-	c = chunkqueue_init();
-	
-	fn = buffer_init_string("server.c");
-	
-	chunkqueue_append_file(c, fn, 0, 10);
-	chunkqueue_append_file(c, fn, 10, 10);
-	chunkqueue_append_file(c, fn, 20, 10);
-	
-	write_chunkqueue(STDERR_FILENO, c);
-	chunkqueue_reset(c);
-	
-	b = buffer_init();
-	buffer_copy_string(b, "\ntest string mit vielen Zeichen\n");
-	chunkqueue_append_buffer(c, b);
-	
-	write_chunkqueue(STDERR_FILENO, c);
-	chunkqueue_reset(c);
-	
-	chunkqueue_append_file(c, fn, 0, 10);
-	buffer_copy_string(b, "\ntest string mit vielen Zeichen\n");
-	chunkqueue_append_buffer(c, b);
-	chunkqueue_append_file(c, fn, 10, 10);
-	chunkqueue_append_file(c, fn, 20, 10);
-	chunkqueue_append_file(c, fn, 50, 40);
-	
-	write_chunkqueue(STDERR_FILENO, c);
-	chunkqueue_reset(c);
-	
-	chunkqueue_free(c);
-	
-	return 0;
-}
-#endif
