@@ -18,11 +18,13 @@
 #include "response.h"
 #include "network.h"
 #include "http_chunk.h"
-#include "file_cache.h"
+#include "stat_cache.h"
 #include "joblist.h"
 
 #include "plugin.h"
 
+#include "inet_ntop_cache.h"
+
 #ifdef USE_OPENSSL
 # include <openssl/ssl.h> 
 # include <openssl/err.h> 
@@ -34,6 +36,10 @@
 
 #include "sys-socket.h"
 
+typedef struct {
+	        PLUGIN_DATA;
+} plugin_data;
+
 static connection *connections_get_new_connection(server *srv) {
 	connections *conns = srv->conns;
 	size_t i;
@@ -211,7 +217,7 @@ static int connection_handle_read(server *srv, connection *con) {
 		
 #ifdef USE_OPENSSL
 		if (srv_sock->is_ssl) {
-			int r;
+			int r, ssl_err;
 			
 			switch ((r = SSL_get_error(con->ssl, len))) {
 			case SSL_ERROR_WANT_READ:
@@ -235,8 +241,15 @@ static int connection_handle_read(server *srv, connection *con) {
 				
 				/* fall thourgh */
 			default:
-				log_error_write(srv, __FILE__, __LINE__, "sds", "SSL:", 
-						r, ERR_error_string(ERR_get_error(), NULL));
+				ssl_err = ERR_get_error();
+				switch(ssl_err) {
+				case SSL_F_SSL23_GET_CLIENT_HELLO:
+					/* a unencrypted HTTP request on a HTTPS socket. Do a redirect to the right location */
+				default:
+					log_error_write(srv, __FILE__, __LINE__, "sds", "SSL:", 
+							r, ERR_error_string(ssl_err, NULL));
+					break;
+				}
 				break;
 			}
 		} else {
@@ -295,203 +308,157 @@ static int connection_handle_read(server *srv, connection *con) {
 }
 
 static int connection_handle_write_prepare(server *srv, connection *con) {
-	struct stat st;
-	int s_len;
-	
-	switch(con->mode) {
-	case DIRECT:
-		switch(con->http_status) {
-		case 400: /* class: header + custom body */
-		case 401:
-		case 403:
-		case 404:
-		case 408:
-		case 411:
-		case 416:
-		case 500:
-		case 501:
-		case 503:
-		case 505: {
-			con->file_finished = 1;
-				
-			/* rewrite the filename */
-			
-			/* FIXME: use con.physical.errorfile
-			 * 
-			 * 
-			 */
-			buffer_reset(con->physical.path);
-			
-			if (con->conf.errorfile_prefix->used) {
-				buffer_copy_string_buffer(con->physical.path, con->conf.errorfile_prefix);
-				buffer_append_string(con->physical.path, get_http_status_body_name(con->http_status));
+	if (con->mode == DIRECT) {
+		/* static files */
+		switch(con->request.http_method) {
+		case HTTP_METHOD_GET:
+		case HTTP_METHOD_POST:
+		case HTTP_METHOD_HEAD:
+		case HTTP_METHOD_PUT:
+		case HTTP_METHOD_MKCOL:
+		case HTTP_METHOD_DELETE:
+		case HTTP_METHOD_COPY:
+		case HTTP_METHOD_MOVE:
+		case HTTP_METHOD_PROPFIND:
+		case HTTP_METHOD_PROPPATCH:
+			break;
+		case HTTP_METHOD_OPTIONS:
+			if (con->uri.path->ptr[0] != '*') {
+				response_header_insert(srv, con, CONST_STR_LEN("Allow"), CONST_STR_LEN("OPTIONS, GET, HEAD, POST"));
+
+				con->http_status = 200;
+				con->file_finished = 1;
+
+				chunkqueue_reset(con->write_queue);
 			}
-			
-			if ((con->physical.path->used <= 1) || 
-			    (-1 == (stat(con->physical.path->ptr, &st)))) {
-				buffer *b;
-				
-				buffer_reset(con->physical.path);
-				
-				b = chunkqueue_get_append_buffer(con->write_queue);
-				
-				/* build default error-page */
-				buffer_copy_string(b, 
-					      "<?xml version=\"1.0\" encoding=\"iso-8859-1\"?>\n"
-					      "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\"\n"
-					      "         \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">\n"
-					      "<html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"en\" lang=\"en\">\n"
-					      " <head>\n"
-					      "  <title>");
-				buffer_append_long(b, con->http_status);
-				buffer_append_string(b, " - ");
-				buffer_append_string(b, get_http_status_name(con->http_status));
-				
-				buffer_append_string(b,
-					      "</title>\n"
-					      " </head>\n"
-					      " <body>\n"
-					      "  <h1>");
-				buffer_append_long(b, con->http_status);
-				buffer_append_string(b, " - ");
-				buffer_append_string(b, get_http_status_name(con->http_status));
-				
-				buffer_append_string(b,"</h1>\n" 
-					      " </body>\n"
-					      "</html>\n"
-					      );
-				
-				response_header_overwrite(srv, con, CONST_STR_LEN("Content-Type"), CONST_STR_LEN("text/html"));
-			} else {
-				/* get content-type */
-				size_t k;
-				s_len = con->physical.path->used - 1;
-				
-				for (k = 0; k < con->conf.mimetypes->used; k++) {
-					data_string *ds = (data_string *)con->conf.mimetypes->data[k];
-					int ct_len = ds->key->used - 1;
-					
-					if (s_len < ct_len ||
-					    ds->key->used == 0) continue;
-					
-					if (0 == strncmp(con->physical.path->ptr + s_len - ct_len, ds->key->ptr, ct_len)) {
-						response_header_overwrite(srv, con, CONST_STR_LEN("Content-Type"), CONST_BUF_LEN(ds->value));
-						break;
-					}
-				}
-				
-				if (k == con->conf.mimetypes->used) {
-					/* the error message should be HTML */
-					response_header_overwrite(srv, con, CONST_STR_LEN("Content-Type"), CONST_STR_LEN("text/html"));
-				}
+			break;
+		default:
+			switch(con->http_status) {
+			case 400: /* bad request */
+			case 505: /* unknown protocol */
+			case 207: /* this was webdav */
+				break;
+			default:
+				con->http_status = 501;
+				break;
 			}
+			break;
 		}
-			/* fall through */
+	}
+	
+	if (con->http_status == 0) {
+		con->http_status = 403;
+	}
+	
+	switch(con->http_status) {
+	case 400: /* class: header + custom body */
+	case 401:
+	case 403:
+	case 404:
+	case 408:
+	case 411:
+	case 416:
+	case 500:
+	case 501:
+	case 503:
+	case 505: 
+		if (con->mode != DIRECT) break;
+		
+		con->file_finished = 0;
+		
+		buffer_reset(con->physical.path);
+				
+		/* try to send static errorfile */
+		if (!buffer_is_empty(con->conf.errorfile_prefix)) {
+			stat_cache_entry *sce = NULL;
 			
-		case 200: /* class: header + body */
-			if (con->physical.path->used) {
+			buffer_copy_string_buffer(con->physical.path, con->conf.errorfile_prefix);
+			buffer_append_string(con->physical.path, get_http_status_body_name(con->http_status));
+			
+			if (HANDLER_ERROR != stat_cache_get_entry(srv, con, con->physical.path, &sce)) {
 				con->file_finished = 1;
 				
-				if (HANDLER_GO_ON != file_cache_get_entry(srv, con, con->physical.path, &(con->fce))) {
-					log_error_write(srv, __FILE__, __LINE__, "sb",
-							strerror(errno), con->physical.path);
-					
-					connection_set_state(srv, con, CON_STATE_ERROR);
-					
-					return -1;
-				}
-				
-				if (S_ISREG(con->fce->st.st_mode)) {
-					if (con->request.http_method == HTTP_METHOD_GET ||
-					    con->request.http_method == HTTP_METHOD_POST) {
-						http_chunk_append_file(srv, con, con->physical.path, 0, con->fce->st.st_size);
-						con->response.content_length = http_chunkqueue_length(srv, con);
-					} else if (con->request.http_method == HTTP_METHOD_HEAD) {
-						con->response.content_length = con->fce->st.st_size;
-					} else {
-						connection_set_state(srv, con, CON_STATE_ERROR);
-						return -1;
-					}
-					
-					http_response_write_header(srv, con, 
-								   con->response.content_length, 
-								   con->fce->st.st_mtime);
-					
-					
-				} else {
-					/* why the heck ? */
-					
-					log_error_write(srv, __FILE__, __LINE__, "sb",
-							"connection closed: no regular-file to send:", 
-							con->physical.path);
-					
-					con->file_finished = 1;
-				}
-			} else {
-				if (con->file_finished) {
-					con->response.content_length = (ssize_t)http_chunkqueue_length(srv, con);
-				}
-				
-				/* disable keep-alive if size-info for the body is missing */
-				if ((con->parsed_response & HTTP_CONTENT_LENGTH) &&
-				    ((con->response.transfer_encoding & HTTP_TRANSFER_ENCODING_CHUNKED) == 0)) {
-					con->keep_alive = 0;
-				}
-				
-				if (con->request.http_method == HTTP_METHOD_HEAD) {
-					chunkqueue_reset(con->write_queue);
-				}
-				
-				http_response_write_header(srv, con, 
-							   con->response.content_length, 
-							   0);
-				
+				http_chunk_append_file(srv, con, con->physical.path, 0, sce->st.st_size);
 			}
-			break;
-			
-		case 206: /* write_queue is already prepared */
-			http_response_write_header(srv, con, 
-						   con->response.content_length,
-						   0);
-			con->file_finished = 1;
-			break;
-		case 302:
-			con->file_finished = 1;
-			
-			con->response.content_length = (ssize_t)http_chunkqueue_length(srv, con);
-				
-			http_response_write_header(srv, con, 
-						   con->response.content_length, 
-						   0);
-			
-			break;
-		case 205: /* class: header only */
-		case 301:
-		case 304:
-		default:
-			/* disable chunked encoding again as we have no body */
-			con->response.transfer_encoding &= ~HTTP_TRANSFER_ENCODING_CHUNKED;
-			chunkqueue_reset(con->write_queue);
-			
-			http_response_write_header(srv, con, 0, 0);
-			con->file_finished = 1;
-			break;
 		}
 		
+		if (!con->file_finished) {			
+			buffer *b;
+			
+			buffer_reset(con->physical.path);
+			
+			con->file_finished = 1;
+			b = chunkqueue_get_append_buffer(con->write_queue);
+				
+			/* build default error-page */
+			buffer_copy_string(b, 
+					   "<?xml version=\"1.0\" encoding=\"iso-8859-1\"?>\n"
+					   "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\"\n"
+					   "         \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">\n"
+					   "<html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"en\" lang=\"en\">\n"
+					   " <head>\n"
+					   "  <title>");
+			buffer_append_long(b, con->http_status);
+			buffer_append_string(b, " - ");
+			buffer_append_string(b, get_http_status_name(con->http_status));
+			
+			buffer_append_string(b,
+					     "</title>\n"
+					     " </head>\n"
+					     " <body>\n"
+					     "  <h1>");
+			buffer_append_long(b, con->http_status);
+			buffer_append_string(b, " - ");
+			buffer_append_string(b, get_http_status_name(con->http_status));
+			
+			buffer_append_string(b,"</h1>\n" 
+					     " </body>\n"
+					     "</html>\n"
+					     );
+			
+			response_header_overwrite(srv, con, CONST_STR_LEN("Content-Type"), CONST_STR_LEN("text/html"));
+		}
+		/* fall through */
+	case 207:
+	case 200: /* class: header + body */
 		break;
+		
+	case 206: /* write_queue is already prepared */
+	case 302:
+		con->file_finished = 1;
+		
+		break;
+	case 205: /* class: header only */
+	case 301:
+	case 304:
 	default:
-		if (con->request.http_method == HTTP_METHOD_HEAD ||
-		    con->http_status == 301 ||
-		    con->http_status == 304 ||
-		    con->http_status == 205) {
-			/* remove possible chunks */
-			con->response.transfer_encoding &= ~HTTP_TRANSFER_ENCODING_CHUNKED;
-			chunkqueue_reset(con->write_queue);
+		/* disable chunked encoding again as we have no body */
+		con->response.transfer_encoding &= ~HTTP_TRANSFER_ENCODING_CHUNKED;
+		chunkqueue_reset(con->write_queue);
+		
+		con->file_finished = 1;
+		break;
+	}
+	
+
+	if (con->file_finished) {
+		/* we have all the content and chunked encoding is not used, set a content-length */ 
+		
+		if ((con->response.transfer_encoding & HTTP_TRANSFER_ENCODING_CHUNKED) == 0) {
+			buffer_copy_off_t(srv->tmp_buf, chunkqueue_length(con->write_queue));
+		
+			response_header_overwrite(srv, con, CONST_STR_LEN("Content-Length"), CONST_BUF_LEN(srv->tmp_buf));
+		}
+	} else {
+		/* disable keep-alive if size-info for the body is missing */
+		if ((con->parsed_response & HTTP_CONTENT_LENGTH) &&
+		    ((con->response.transfer_encoding & HTTP_TRANSFER_ENCODING_CHUNKED) == 0)) {
+			con->keep_alive = 0;
 		}
 		
 		if (0 == (con->parsed_response & HTTP_CONNECTION)) {
-			/* (f)cgi did'nt send Connection: header 
-			 * 
+			/* (f)cgi did'nt send Connection: header
+			 *                          
 			 * shall we ?
 			 */
 			if (((con->response.transfer_encoding & HTTP_TRANSFER_ENCODING_CHUNKED) == 0) &&
@@ -508,13 +475,14 @@ static int connection_handle_write_prepare(server *srv, connection *con) {
 				/* FIXME: we have to drop the Connection: Header from the subrequest */
 			}
 		}
-		
-		con->response.content_length = (ssize_t)http_chunkqueue_length(srv, con);
-		http_response_write_basic_header(srv, con);
-			
-		break;
 	}
 	
+	if (con->request.http_method == HTTP_METHOD_HEAD) {
+		chunkqueue_reset(con->write_queue);
+	}
+
+	http_response_write_header(srv, con);
+		
 	return 0;
 }
 
@@ -582,6 +550,7 @@ connection *connection_init(server *srv) {
 	
 	CLEAN(physical.doc_root);
 	CLEAN(physical.path);
+	CLEAN(physical.basedir);
 	CLEAN(physical.rel_path);
 	CLEAN(physical.etag);
 	CLEAN(parse_request);
@@ -589,6 +558,7 @@ connection *connection_init(server *srv) {
 	CLEAN(authed_user);
 	CLEAN(server_name);
 	CLEAN(error_handler);
+	CLEAN(dst_addr_buf);
 	
 #undef CLEAN
 	con->write_queue = chunkqueue_init();
@@ -601,6 +571,7 @@ connection *connection_init(server *srv) {
 	
 	con->plugin_ctx = calloc(srv->plugins.used + 1, sizeof(void *));
 	
+	con->cond_cache = calloc(srv->config_context->used, sizeof(cond_cache_t));
 	config_setup_connection(srv, con);
 	
 	return con;
@@ -640,6 +611,7 @@ void connections_free(server *srv) {
 		
 		CLEAN(physical.doc_root);
 		CLEAN(physical.path);
+		CLEAN(physical.basedir);
 		CLEAN(physical.etag);
 		CLEAN(physical.rel_path);
 		CLEAN(parse_request);
@@ -647,8 +619,10 @@ void connections_free(server *srv) {
 		CLEAN(authed_user);
 		CLEAN(server_name);
 		CLEAN(error_handler);
+		CLEAN(dst_addr_buf);
 #undef CLEAN
 		free(con->plugin_ctx);
+		free(con->cond_cache);
 		
 		free(con);
 	}
@@ -708,6 +682,7 @@ int connection_reset(server *srv, connection *con) {
 	
 	CLEAN(physical.doc_root);
 	CLEAN(physical.path);
+	CLEAN(physical.basedir);
 	CLEAN(physical.rel_path);
 	CLEAN(physical.etag);
 	
@@ -716,6 +691,7 @@ int connection_reset(server *srv, connection *con) {
 	CLEAN(authed_user);
 	CLEAN(server_name);
 	CLEAN(error_handler);
+	CLEAN(dst_addr_buf);
 #undef CLEAN	
 	
 #define CLEAN(x) \
@@ -737,16 +713,28 @@ int connection_reset(server *srv, connection *con) {
 	array_reset(con->environment);
 	
 	chunkqueue_reset(con->write_queue);
-	
-	if (con->fce) {
-		file_cache_entry_release(srv, con, con->fce);
-		con->fce = NULL;
-	}
-	
+
+	/* the plugins should cleanup themself */	
 	for (i = 0; i < srv->plugins.used; i++) {
-		con->plugin_ctx[0] = NULL;
+		plugin *p = ((plugin **)(srv->plugins.ptr))[i];
+		plugin_data *pd = p->data;
+
+		if (con->plugin_ctx[pd->id] != NULL) {
+			log_error_write(srv, __FILE__, __LINE__, "sb", "missing cleanup in", p->name);
+		}
+
+		con->plugin_ctx[pd->id] = NULL;
 	}
 	
+#if COND_RESULT_UNSET
+	for (i = srv->config_context->used - 1; i >= 0; i --) {
+		con->cond_cache[i].result = COND_RESULT_UNSET;
+		con->cond_cache[i].patterncount = 0;
+	}
+#else
+	memset(con->cond_cache, 0, sizeof(cond_cache_t) * srv->config_context->used);
+#endif
+	
 	con->header_len = 0;
 	con->in_error_handler = 0;
 	
@@ -817,7 +805,7 @@ int connection_handle_read_state(server *srv, connection *con)  {
 	for (c = cq->first; c; c = cq->first) {
 		assert(c != c->next);
 		
-		if (c->data.mem->used == 0) {
+		if (c->mem->used == 0) {
 			cq->first = c->next;
 			c->next = cq->unused;
 			cq->unused = c;
@@ -842,8 +830,8 @@ int connection_handle_read_state(server *srv, connection *con)  {
 		if (con->request.request->used == 0) {
 			buffer b;
 			
-			b.ptr = c->data.mem->ptr + c->offset;
-			b.used = c->data.mem->used - c->offset;
+			b.ptr = c->mem->ptr + c->offset;
+			b.used = c->mem->used - c->offset;
 			
 			if (NULL != (h_term = buffer_search_rnrn(&b))) {
 				/* \r\n\r\n found
@@ -858,8 +846,8 @@ int connection_handle_read_state(server *srv, connection *con)  {
 				c->offset += h_term - b.ptr + 4;
 			} else {
 				/* not found, copy everything */
-				buffer_copy_string_len(con->request.request, c->data.mem->ptr + c->offset, c->data.mem->used - c->offset - 1);
-				c->offset = c->data.mem->used - 1;
+				buffer_copy_string_len(con->request.request, c->mem->ptr + c->offset, c->mem->used - c->offset - 1);
+				c->offset = c->mem->used - 1;
 			}
 		} else {
 			/* have to take care of overlapping header terminators */
@@ -868,36 +856,36 @@ int connection_handle_read_state(server *srv, connection *con)  {
 			char *s  = con->request.request->ptr;
 			buffer b;
 			
-			b.ptr = c->data.mem->ptr + c->offset;
-			b.used = c->data.mem->used - c->offset;
+			b.ptr = c->mem->ptr + c->offset;
+			b.used = c->mem->used - c->offset;
 			
 			if (con->request.request->used - 1 > 3 &&
-			    c->data.mem->used > 1 &&
+			    c->mem->used > 1 &&
 			    s[l-2] == '\r' &&
 			    s[l-1] == '\n' &&
 			    s[l-0] == '\r' &&
-			    c->data.mem->ptr[0] == '\n') {
-				buffer_append_string_len(con->request.request, c->data.mem->ptr + c->offset, 1);
+			    c->mem->ptr[0] == '\n') {
+				buffer_append_string_len(con->request.request, c->mem->ptr + c->offset, 1);
 				c->offset += 1;
 				
 				h_term = con->request.request->ptr;
 			} else if (con->request.request->used - 1 > 2 &&
-				   c->data.mem->used > 2 &&
+				   c->mem->used > 2 &&
 				   s[l-1] == '\r' &&
 				   s[l-0] == '\n' &&
-				   c->data.mem->ptr[0] == '\r' &&
-				   c->data.mem->ptr[1] == '\n') {
-				buffer_append_string_len(con->request.request, c->data.mem->ptr + c->offset, 2);
+				   c->mem->ptr[0] == '\r' &&
+				   c->mem->ptr[1] == '\n') {
+				buffer_append_string_len(con->request.request, c->mem->ptr + c->offset, 2);
 				c->offset += 2;
 				
 				h_term = con->request.request->ptr;
 			} else if (con->request.request->used - 1 > 1 &&
-				   c->data.mem->used > 3 &&
+				   c->mem->used > 3 &&
 				   s[l-0] == '\r' &&
-				   c->data.mem->ptr[0] == '\n' &&
-				   c->data.mem->ptr[1] == '\r' &&
-				   c->data.mem->ptr[2] == '\n') {
-				buffer_append_string_len(con->request.request, c->data.mem->ptr + c->offset, 3);
+				   c->mem->ptr[0] == '\n' &&
+				   c->mem->ptr[1] == '\r' &&
+				   c->mem->ptr[2] == '\n') {
+				buffer_append_string_len(con->request.request, c->mem->ptr + c->offset, 3);
 				c->offset += 3;
 				
 				h_term = con->request.request->ptr;
@@ -907,19 +895,19 @@ int connection_handle_read_state(server *srv, connection *con)  {
 				 */
 				
 				buffer_append_string_len(con->request.request, 
-						       c->data.mem->ptr + c->offset, 
+						       c->mem->ptr + c->offset, 
 						       c->offset + h_term - b.ptr + 4);
 				
 				/* the buffer has been read up to the terminator */
 				c->offset += h_term - b.ptr + 4;
 			} else {
 				/* not found, copy everything */
-				buffer_append_string_len(con->request.request, c->data.mem->ptr + c->offset, c->data.mem->used - c->offset - 1);
-				c->offset = c->data.mem->used - 1;
+				buffer_append_string_len(con->request.request, c->mem->ptr + c->offset, c->mem->used - c->offset - 1);
+				c->offset = c->mem->used - 1;
 			}
 		}
 		
-		if (c->offset + 1 == c->data.mem->used) {
+		if (c->offset + 1 == c->mem->used) {
 			/* chunk is empty, move it to unused */
 			cq->first = c->next;
 			c->next = cq->unused;
@@ -945,17 +933,17 @@ int connection_handle_read_state(server *srv, connection *con)  {
 			weWant = con->request.content_length - (con->request.content->used ? con->request.content->used - 1 : 0);
 			/* without the terminating \0 */
 			
-			assert(c->data.mem->used);
+			assert(c->mem->used);
 			
-			weHave = c->data.mem->used - c->offset - 1;
+			weHave = c->mem->used - c->offset - 1;
 				
 			toRead = weHave > weWant ? weWant : weHave;
 			
-			buffer_append_string_len(con->request.content, c->data.mem->ptr + c->offset, toRead);
+			buffer_append_string_len(con->request.content, c->mem->ptr + c->offset, toRead);
 			
 			c->offset += toRead;
 			
-			if (c->offset + 1 >= c->data.mem->used) {
+			if (c->offset + 1 >= c->mem->used) {
 				/* chunk is empty, move it to unused */
 				
 				cq->first = c->next;
@@ -1083,7 +1071,6 @@ handler_t connection_handle_fdevent(void *s, void *context, int revents) {
 
 
 connection *connection_accept(server *srv, server_socket *srv_socket) {
-	int accepted_requests = 0;
 	/* accept everything */
 
 	/* search an empty place */
@@ -1097,7 +1084,7 @@ connection *connection_accept(server *srv, server_socket *srv_socket) {
 	if (-1 == (cnt = accept(srv_socket->fd, (struct sockaddr *) &cnt_addr, &cnt_len))) {
 		if ((errno != EAGAIN) &&
 		    (errno != EINTR)) {
-			log_error_write(srv, __FILE__, __LINE__, "ss", "accept failed: ", strerror(errno));
+			log_error_write(srv, __FILE__, __LINE__, "ssd", "accept failed:", strerror(errno), errno);
 		}
 		return NULL;
 	} else {
@@ -1105,7 +1092,6 @@ connection *connection_accept(server *srv, server_socket *srv_socket) {
 		
 		srv->cur_fds++;
 		
-		accepted_requests++;
 		/* ok, we have the connection, register it */
 #if 0
 		log_error_write(srv, __FILE__, __LINE__, "sd",
@@ -1126,6 +1112,7 @@ connection *connection_accept(server *srv, server_socket *srv_socket) {
 		
 		con->connection_start = srv->cur_ts;
 		con->dst_addr = cnt_addr;
+		buffer_copy_string(con->dst_addr_buf, inet_ntop_cache_get_ip(srv, &(con->dst_addr)));
 		con->srv_socket = srv_socket;
 		
 		if (-1 == (fdevent_fcntl_set(srv->ev, con->fd))) {
@@ -1304,7 +1291,11 @@ int connection_state_machine(server *srv, connection *con) {
 						"state for fd", con->fd, connection_get_state(con->state));
 			}
 			
-			connection_handle_write_prepare(srv, con);
+			if (-1 == connection_handle_write_prepare(srv, con)) {
+				connection_set_state(srv, con, CON_STATE_ERROR);
+				
+				break;
+			}
 			
 			connection_set_state(srv, con, CON_STATE_WRITE);
 			break;
