@@ -1,10 +1,12 @@
 #include <string.h>
 #include <errno.h>
+#include <time.h>
 
 #include "base.h"
 #include "array.h"
 #include "buffer.h"
 #include "log.h"
+#include "etag.h"
 
 /*
  * This was 'borrowed' from tcpdump.
@@ -158,8 +160,8 @@ int http_response_redirect_to_directory(server *srv, connection *con) {
 		case AF_INET:
 			if (NULL == (he = gethostbyaddr((char *)&our_addr.ipv4.sin_addr, sizeof(struct in_addr), AF_INET))) {
 				log_error_write(srv, __FILE__, __LINE__,
-						"SSSS", "NOTICE: gethostbyaddr failed: ",
-						hstrerror(h_errno), ", using ip-address instead");
+						"SdSS", "NOTICE: gethostbyaddr failed: ",
+						h_errno, ", using ip-address instead");
 				
 				buffer_append_string(o, inet_ntoa(our_addr.ipv4.sin_addr));
 			} else {
@@ -196,3 +198,119 @@ int http_response_redirect_to_directory(server *srv, connection *con) {
 	return 0;
 }
 
+buffer * strftime_cache_get(server *srv, time_t last_mod) {
+	struct tm *tm;
+	size_t i;
+		
+	for (i = 0; i < FILE_CACHE_MAX; i++) {
+		/* found cache-entry */
+		if (srv->mtime_cache[i].mtime == last_mod) return srv->mtime_cache[i].str;
+				
+		/* found empty slot */
+		if (srv->mtime_cache[i].mtime == 0) break;
+	}
+	
+	if (i == FILE_CACHE_MAX) {
+		i = 0;
+	}
+		
+	srv->mtime_cache[i].mtime = last_mod;
+	buffer_prepare_copy(srv->mtime_cache[i].str, 1024);
+	tm = gmtime(&(srv->mtime_cache[i].mtime));
+	srv->mtime_cache[i].str->used = strftime(srv->mtime_cache[i].str->ptr, 
+						 srv->mtime_cache[i].str->size - 1,
+						 "%a, %d %b %Y %H:%M:%S GMT", tm);
+	srv->mtime_cache[i].str->used++;
+	
+	return srv->mtime_cache[i].str;
+}
+
+
+int http_response_handle_cachable(server *srv, connection *con, buffer *mtime) {
+	/*
+	 * 14.26 If-None-Match
+	 *    [...]
+	 *    If none of the entity tags match, then the server MAY perform the
+	 *    requested method as if the If-None-Match header field did not exist,
+	 *    but MUST also ignore any If-Modified-Since header field(s) in the
+	 *    request. That is, if no entity tags match, then the server MUST NOT
+	 *    return a 304 (Not Modified) response.
+	 */
+	
+	/* last-modified handling */
+	if (con->request.http_if_none_match) {
+		if (etag_is_equal(con->physical.etag, con->request.http_if_none_match)) {
+			if (con->request.http_method == HTTP_METHOD_GET || 
+			    con->request.http_method == HTTP_METHOD_HEAD) {
+				
+				/* check if etag + last-modified */
+				if (con->request.http_if_modified_since) {
+					
+					size_t used_len;
+					char *semicolon;
+					
+					if (NULL == (semicolon = strchr(con->request.http_if_modified_since, ';'))) {
+						used_len = strlen(con->request.http_if_modified_since);
+					} else {
+						used_len = semicolon - con->request.http_if_modified_since;
+					}
+					
+					if (0 == strncmp(con->request.http_if_modified_since, mtime->ptr, used_len)) {
+						con->http_status = 304;
+						return HANDLER_FINISHED;
+					} else {
+						char buf[sizeof("Sat, 23 Jul 2005 21:20:01 GMT")];
+
+						/* convert to timestamp */
+						if (used_len < sizeof(buf) - 1) {
+							time_t t_header, t_file;
+							struct tm tm;
+							
+							strncpy(buf, con->request.http_if_modified_since, used_len);
+							buf[used_len] = '\0';
+							
+							strptime(buf, "%a, %d %b %Y %H:%M:%S GMT", &tm);
+							t_header = mktime(&tm);
+							
+							strptime(mtime->ptr, "%a, %d %b %Y %H:%M:%S GMT", &tm);
+							t_file = mktime(&tm);
+
+							if (t_file > t_header) {
+								con->http_status = 304;
+								return HANDLER_FINISHED;
+							}
+						} else {
+							log_error_write(srv, __FILE__, __LINE__, "ss", 
+									con->request.http_if_modified_since, buf);
+							
+							con->http_status = 412;
+							return HANDLER_FINISHED;
+						}
+					}
+				} else {
+					con->http_status = 304;
+					return HANDLER_FINISHED;
+				}
+			} else {
+				con->http_status = 412;
+				return HANDLER_FINISHED;
+			}
+		}
+	} else if (con->request.http_if_modified_since) {
+		size_t used_len;
+		char *semicolon;
+		
+		if (NULL == (semicolon = strchr(con->request.http_if_modified_since, ';'))) {
+			used_len = strlen(con->request.http_if_modified_since);
+		} else {
+			used_len = semicolon - con->request.http_if_modified_since;
+		}
+		
+		if (0 == strncmp(con->request.http_if_modified_since, mtime->ptr, used_len)) {
+			con->http_status = 304;
+			return HANDLER_FINISHED;
+		}
+	}
+
+	return HANDLER_GO_ON;
+}
