@@ -16,6 +16,7 @@
 
 #include <fcntl.h>
 #include <stdlib.h>
+#include <stdio.h>
 #include <string.h>
 #include <time.h>
 #include <errno.h>
@@ -441,22 +442,59 @@ static int http_auth_basic_password_compare(server *srv, mod_auth_plugin_data *p
 		}
 	} else if (p->conf.auth_backend == AUTH_BACKEND_HTPASSWD) { 
 #ifdef HAVE_CRYPT	
-		char salt[3];
+		char salt[32];
 		char *crypted;
+		size_t salt_len = 0;
 		/* 
 		 * htpasswd format
 		 * 
 		 * user:crypted password
 		 */
+
+		/* 
+		 *  Algorithm      Salt
+		 *  CRYPT_STD_DES   2-character (Default)
+		 *  CRYPT_EXT_DES   9-character
+		 *  CRYPT_MD5       12-character beginning with $1$
+		 *  CRYPT_BLOWFISH  16-character beginning with $2$
+		 */
+
+		if (password->used < 13 + 1) {
+			fprintf(stderr, "%s.%d\n", __FILE__, __LINE__);
+			return -1;
+		}
+
+		if (password->used == 13 + 1) {
+			/* a simple DES password is 2 + 11 characters */
+			salt_len = 2;
+		} else if (password->ptr[0] == '$' && password->ptr[2] == '$') {
+			char *dollar = NULL;
+		
+			if (NULL == (dollar = strchr(password->ptr + 3, '$'))) {
+				fprintf(stderr, "%s.%d\n", __FILE__, __LINE__);
+				return -1;
+			}
+
+			salt_len = dollar - password->ptr;
+		}
+
+		if (salt_len - 1 > sizeof(salt)) {
+			fprintf(stderr, "%s.%d\n", __FILE__, __LINE__);
+			return -1;
+		}
+
+		strncpy(salt, password->ptr, salt_len);
+
+		salt[salt_len] = '\0';
 		
-		salt[0] = password->ptr[0];
-		salt[1] = password->ptr[1];
-		salt[2] = '\0';
 		crypted = crypt(pw, salt);
 
 		if (0 == strcmp(password->ptr, crypted)) {
 			return 0;
+		} else {
+			fprintf(stderr, "%s.%d\n", __FILE__, __LINE__);
 		}
+	
 #endif	
 	} else if (p->conf.auth_backend == AUTH_BACKEND_PLAIN) { 
 		if (0 == strcmp(password->ptr, pw)) {
@@ -578,6 +616,16 @@ static int http_auth_basic_password_compare(server *srv, mod_auth_plugin_data *p
 			return -1;
 		}
 		
+		if (p->conf.auth_ldap_starttls == 1) {
+	 		if (LDAP_OPT_SUCCESS != (ret = ldap_start_tls_s(ldap, NULL,  NULL))) {
+	 			log_error_write(srv, __FILE__, __LINE__, "ss", "ldap startTLS failed:", ldap_err2string(ret));
+		
+				ldap_unbind_s(ldap);
+				
+				return -1;
+	 		}
+ 		}
+
 		
 		if (LDAP_SUCCESS != (ret = ldap_simple_bind_s(ldap, dn, pw))) {
 			log_error_write(srv, __FILE__, __LINE__, "ss", "ldap:", ldap_err2string(ret));
@@ -636,7 +684,7 @@ int http_auth_basic_check(server *srv, connection *con, mod_auth_plugin_data *p,
 	
 	/* password doesn't match */
 	if (http_auth_basic_password_compare(srv, p, req, username, realm->value, password, pw)) {
-		log_error_write(srv, __FILE__, __LINE__, "sbb", "password doesn't match", con->uri.path, username);
+		log_error_write(srv, __FILE__, __LINE__, "sbb", "password doesn't match for", con->uri.path, username);
 		
 		buffer_free(username);
 		buffer_free(password);
@@ -683,10 +731,6 @@ int http_auth_digest_check(server *srv, connection *con, mod_auth_plugin_data *p
 	char *nc;
 	char *respons;
 	
-	const char *method_get = "GET";
-	const char *method_post = "POST";
-	const char *method_head = "HEAD";
-	
 	char *e, *c;
 	const char *m = NULL;
 	int i;
@@ -796,13 +840,8 @@ int http_auth_digest_check(server *srv, connection *con, mod_auth_plugin_data *p
 				"digest: missing field");
 		return -1;
 	}
-	
-	switch(con->request.http_method) {
-	case HTTP_METHOD_GET: m = method_get; break;
-	case HTTP_METHOD_POST: m = method_post; break;
-	case HTTP_METHOD_HEAD: m = method_head; break;
-	case HTTP_METHOD_UNSET: break;
-	}
+
+	m = get_http_method_name(con->request.http_method);	
 
 	/* password-string == HA1 */
 	password = buffer_init();
