@@ -63,12 +63,12 @@ SETDEFAULTS_FUNC(mod_access_set_defaults) {
 		{ NULL,                          NULL, T_CONFIG_UNSET, T_CONFIG_SCOPE_UNSET }
 	};
 	
-	p->config_storage = malloc(srv->config_context->used * sizeof(specific_config *));
+	p->config_storage = calloc(1, srv->config_context->used * sizeof(specific_config *));
 	
 	for (i = 0; i < srv->config_context->used; i++) {
 		plugin_config *s;
 		
-		s = malloc(sizeof(plugin_config));
+		s = calloc(1, sizeof(plugin_config));
 		s->access_deny    = array_init();
 		
 		cv[0].destination = s->access_deny;
@@ -85,16 +85,16 @@ SETDEFAULTS_FUNC(mod_access_set_defaults) {
 
 #define PATCH(x) \
 	p->conf.x = s->x;
-static int mod_access_patch_connection(server *srv, connection *con, plugin_data *p, const char *stage, size_t stage_len) {
+static int mod_access_patch_connection(server *srv, connection *con, plugin_data *p) {
 	size_t i, j;
+	plugin_config *s = p->config_storage[0];
+
+	PATCH(access_deny);
 	
 	/* skip the first, the global context */
 	for (i = 1; i < srv->config_context->used; i++) {
 		data_config *dc = (data_config *)srv->config_context->data[i];
-		plugin_config *s = p->config_storage[i];
-		
-		/* not our stage */
-		if (!buffer_is_equal_string(dc->comp_key, stage, stage_len)) continue;
+		s = p->config_storage[i];
 		
 		/* condition didn't match */
 		if (!config_check_cond(srv, con, dc)) continue;
@@ -111,34 +111,16 @@ static int mod_access_patch_connection(server *srv, connection *con, plugin_data
 	
 	return 0;
 }
-
-static int mod_access_setup_connection(server *srv, connection *con, plugin_data *p) {
-	plugin_config *s = p->config_storage[0];
-	
-	UNUSED(srv);
-	UNUSED(con);
-		
-	PATCH(access_deny);
-	
-	return 0;
-}
 #undef PATCH
 
 URIHANDLER_FUNC(mod_access_uri_handler) {
 	plugin_data *p = p_d;
 	int s_len;
-	size_t k, i;
+	size_t k;
 	
-	UNUSED(srv);
-
 	if (con->uri.path->used == 0) return HANDLER_GO_ON;
 	
-	mod_access_setup_connection(srv, con, p);
-	for (i = 0; i < srv->config_patches->used; i++) {
-		buffer *patch = srv->config_patches->ptr[i];
-		
-		mod_access_patch_connection(srv, con, p, CONST_BUF_LEN(patch));
-	}
+	mod_access_patch_connection(srv, con, p);
 	
 	s_len = con->uri.path->used - 1;
 	
