@@ -1,6 +1,7 @@
 #include <ctype.h>
 #include <stdlib.h>
 #include <string.h>
+#include <stdio.h>
 
 #include "base.h"
 #include "log.h"
@@ -72,12 +73,12 @@ SETDEFAULTS_FUNC(mod_alias_set_defaults) {
 	
 	if (!p) return HANDLER_ERROR;
 	
-	p->config_storage = malloc(srv->config_context->used * sizeof(specific_config *));
+	p->config_storage = calloc(1, srv->config_context->used * sizeof(specific_config *));
 	
 	for (i = 0; i < srv->config_context->used; i++) {
 		plugin_config *s;
 		
-		s = malloc(sizeof(plugin_config));
+		s = calloc(1, sizeof(plugin_config));
 		s->alias = array_init();	
 		cv[0].destination = s->alias;
 		
@@ -86,6 +87,31 @@ SETDEFAULTS_FUNC(mod_alias_set_defaults) {
 		if (0 != config_insert_values_global(srv, ((data_config *)srv->config_context->data[i])->value, cv)) {
 			return HANDLER_ERROR;
 		}
+		if (s->alias->used >= 2) {
+			const array *a = s->alias;
+			size_t j, k;
+
+			for (j = 0; j < a->used; j ++) {
+				const buffer *prefix = a->data[a->sorted[j]]->key;
+				for (k = j + 1; k < a->used; k ++) {
+					const buffer *key = a->data[a->sorted[k]]->key;
+
+					if (key->used < prefix->used) {
+						break;
+					}
+					if (memcmp(key->ptr, prefix->ptr, prefix->used - 1) != 0) {
+						break;
+					}
+					/* ok, they have same prefix. check position */
+					if (a->sorted[j] < a->sorted[k]) {
+						fprintf(stderr, "url.alias: `%s' will never match as `%s' matched first\n",
+								key->ptr,
+								prefix->ptr);
+						return HANDLER_ERROR;
+					}
+				}
+			}
+		}
 	}
 	
 	return HANDLER_GO_ON;
@@ -93,16 +119,16 @@ SETDEFAULTS_FUNC(mod_alias_set_defaults) {
 
 #define PATCH(x) \
 	p->conf.x = s->x;
-static int mod_alias_patch_connection(server *srv, connection *con, plugin_data *p, const char *stage, size_t stage_len) {
+static int mod_alias_patch_connection(server *srv, connection *con, plugin_data *p) {
 	size_t i, j;
+	plugin_config *s = p->config_storage[0];
+	
+	PATCH(alias);
 	
 	/* skip the first, the global context */
 	for (i = 1; i < srv->config_context->used; i++) {
 		data_config *dc = (data_config *)srv->config_context->data[i];
-		plugin_config *s = p->config_storage[i];
-		
-		/* not our stage */
-		if (!buffer_is_equal_string(dc->comp_key, stage, stage_len)) continue;
+		s = p->config_storage[i];
 		
 		/* condition didn't match */
 		if (!config_check_cond(srv, con, dc)) continue;
@@ -119,33 +145,22 @@ static int mod_alias_patch_connection(server *srv, connection *con, plugin_data
 	
 	return 0;
 }
-
-static int mod_alias_setup_connection(server *srv, connection *con, plugin_data *p) {
-	plugin_config *s = p->config_storage[0];
-	UNUSED(srv);
-	UNUSED(con);
-		
-	PATCH(alias);
-	
-	return 0;
-}
 #undef PATCH
 
-URIHANDLER_FUNC(mod_alias_docroot_handler) {
+PHYSICALPATH_FUNC(mod_alias_physical_handler) {
 	plugin_data *p = p_d;
-	int uri_len;
-	size_t k, i;
+	int uri_len, basedir_len;
+	char *uri_ptr;
+	size_t k;
 	
-	if (con->uri.path->used == 0) return HANDLER_GO_ON;
+	if (con->physical.path->used == 0) return HANDLER_GO_ON;
 	
-	mod_alias_setup_connection(srv, con, p);
-	for (i = 0; i < srv->config_patches->used; i++) {
-		buffer *patch = srv->config_patches->ptr[i];
-		
-		mod_alias_patch_connection(srv, con, p, CONST_BUF_LEN(patch));
-	}
+	mod_alias_patch_connection(srv, con, p);
 	
-	uri_len = con->uri.path->used - 1;
+	/* not to include the tailing slash */
+	basedir_len = (con->physical.basedir->used - 1) - 1;
+	uri_len = con->physical.path->used - 1 - basedir_len;
+	uri_ptr = con->physical.path->ptr + basedir_len;
 	
 	for (k = 0; k < p->conf.alias->used; k++) {
 		data_string *ds = (data_string *)p->conf.alias->data[k];
@@ -154,11 +169,13 @@ URIHANDLER_FUNC(mod_alias_docroot_handler) {
 		if (alias_len > uri_len) continue;
 		if (ds->key->used == 0) continue;
 		
-		if (0 == strncmp(con->uri.path->ptr, ds->key->ptr, alias_len)) {
+		if (0 == strncmp(uri_ptr, ds->key->ptr, alias_len)) {
 			/* matched */
 			
-			buffer_copy_string_buffer(con->physical.doc_root, ds->value);
-			buffer_copy_string(con->physical.rel_path, con->uri.path->ptr + alias_len);
+			buffer_copy_string_buffer(con->physical.basedir, ds->value);
+			buffer_copy_string_buffer(srv->tmp_buf, ds->value);
+			buffer_append_string(srv->tmp_buf, uri_ptr + alias_len);
+			buffer_copy_string_buffer(con->physical.path, srv->tmp_buf);
 			
 			return HANDLER_GO_ON;
 		}
@@ -175,7 +192,7 @@ int mod_alias_plugin_init(plugin *p) {
 	p->name        = buffer_init_string("alias");
 	
 	p->init           = mod_alias_init;
-	p->handle_docroot = mod_alias_docroot_handler;
+	p->handle_physical= mod_alias_physical_handler;
 	p->set_defaults   = mod_alias_set_defaults;
 	p->cleanup        = mod_alias_free;
 	
