@@ -30,6 +30,10 @@
 
 #include "plugin.h"
 
+#ifdef HAVE_SYS_FILIO_H
+# include <sys/filio.h>
+#endif
+
 enum {EOL_UNSET, EOL_N, EOL_RN};
 
 typedef struct {
@@ -147,12 +151,12 @@ SETDEFAULTS_FUNC(mod_fastcgi_set_defaults) {
 
 	if (!p) return HANDLER_ERROR;
 	
-	p->config_storage = malloc(srv->config_context->used * sizeof(specific_config *));
+	p->config_storage = calloc(1, srv->config_context->used * sizeof(specific_config *));
 	
 	for (i = 0; i < srv->config_context->used; i++) {
 		plugin_config *s;
 		
-		s = malloc(sizeof(plugin_config));
+		s = calloc(1, sizeof(plugin_config));
 		assert(s);
 		
 		s->cgi    = array_init();
@@ -862,9 +866,15 @@ static int cgi_create_env(server *srv, connection *con, plugin_data *p, buffer *
 				buffer_prepare_append(p->tmp_buf, ds->key->used + 2);
 				
 				for (j = 0; j < ds->key->used - 1; j++) {
-					p->tmp_buf->ptr[p->tmp_buf->used++] = 
-						isalpha((unsigned char)ds->key->ptr[j]) ? 
-						toupper((unsigned char)ds->key->ptr[j]) : '_';
+					char cr = '_';
+					if (light_isalpha(ds->key->ptr[j])) {
+						/* upper-case */
+						cr = ds->key->ptr[j] & ~32;
+					} else if (light_isdigit(ds->key->ptr[j])) {
+						/* copy */
+						cr = ds->key->ptr[j];
+					}
+					p->tmp_buf->ptr[p->tmp_buf->used++] = cr;
 				}
 				p->tmp_buf->ptr[p->tmp_buf->used++] = '\0';
 				
@@ -1001,16 +1011,16 @@ static int cgi_create_env(server *srv, connection *con, plugin_data *p, buffer *
 
 #define PATCH(x) \
 	p->conf.x = s->x;
-static int mod_cgi_patch_connection(server *srv, connection *con, plugin_data *p, const char *stage, size_t stage_len) {
+static int mod_cgi_patch_connection(server *srv, connection *con, plugin_data *p) {
 	size_t i, j;
+	plugin_config *s = p->config_storage[0];
+	
+	PATCH(cgi);
 	
 	/* skip the first, the global context */
 	for (i = 1; i < srv->config_context->used; i++) {
 		data_config *dc = (data_config *)srv->config_context->data[i];
-		plugin_config *s = p->config_storage[i];
-		
-		/* not our stage */
-		if (!buffer_is_equal_string(dc->comp_key, stage, stage_len)) continue;
+		s = p->config_storage[i];
 		
 		/* condition didn't match */
 		if (!config_check_cond(srv, con, dc)) continue;
@@ -1027,31 +1037,16 @@ static int mod_cgi_patch_connection(server *srv, connection *con, plugin_data *p
 	
 	return 0;
 }
-
-static int mod_cgi_setup_connection(server *srv, connection *con, plugin_data *p) {
-	plugin_config *s = p->config_storage[0];
-	UNUSED(srv);
-	UNUSED(con);
-		
-	PATCH(cgi);
-	
-	return 0;
-}
 #undef PATCH
 
 URIHANDLER_FUNC(cgi_is_handled) {
-	size_t k, s_len, i;
+	size_t k, s_len;
 	plugin_data *p = p_d;
 	buffer *fn = con->physical.path;
 	
 	if (fn->used == 0) return HANDLER_ERROR;
 	
-	mod_cgi_setup_connection(srv, con, p);
-	for (i = 0; i < srv->config_patches->used; i++) {
-		buffer *patch = srv->config_patches->ptr[i];
-		
-		mod_cgi_patch_connection(srv, con, p, CONST_BUF_LEN(patch));
-	}
+	mod_cgi_patch_connection(srv, con, p);
 	
 	s_len = fn->used - 1;
 	
@@ -1067,9 +1062,10 @@ URIHANDLER_FUNC(cgi_is_handled) {
 				con->http_status = 500;
 				
 				buffer_reset(con->physical.path);
+				return HANDLER_FINISHED;
 			}
-			
-			return HANDLER_FINISHED;
+			/* one handler is enough for the request */
+			break;
 		}
 	}
 	
@@ -1133,12 +1129,7 @@ SUBREQUEST_FUNC(mod_cgi_handle_subrequest) {
 #ifndef __WIN32	
 	switch(waitpid(hctx->pid, &status, WNOHANG)) {
 	case 0:
-		/* not finished yet */
-		if (con->file_started) {
-			return HANDLER_GO_ON;
-		} else {
-			return HANDLER_WAIT_FOR_EVENT;
-		}
+		return HANDLER_WAIT_FOR_EVENT;
 	case -1:
 		if (errno == EINTR) return HANDLER_WAIT_FOR_EVENT;
 		
