@@ -13,10 +13,12 @@
 #include "log.h"
 #include "buffer.h"
 #include "response.h"
+#include "stat_cache.h"
 
 #include "plugin.h"
 
 #include "crc32.h"
+#include "etag.h"
 
 #include "config.h"
 
@@ -85,6 +87,8 @@ FREE_FUNC(mod_compress_free) {
 		size_t i;
 		for (i = 0; i < srv->config_context->used; i++) {
 			plugin_config *s = p->config_storage[i];
+
+			if (!s) continue;
 			
 			array_free(s->compress);
 			buffer_free(s->compress_cache_dir);
@@ -111,12 +115,12 @@ SETDEFAULTS_FUNC(mod_compress_setdefaults) {
 		{ NULL,                             NULL, T_CONFIG_UNSET, T_CONFIG_SCOPE_UNSET }
 	};
 	
-	p->config_storage = malloc(srv->config_context->used * sizeof(specific_config *));
+	p->config_storage = calloc(1, srv->config_context->used * sizeof(specific_config *));
 	
 	for (i = 0; i < srv->config_context->used; i++) {
 		plugin_config *s;
 		
-		s = malloc(sizeof(plugin_config));
+		s = calloc(1, sizeof(plugin_config));
 		s->compress_cache_dir = buffer_init();
 		s->compress = array_init();
 		s->compress_max_filesize = 0;
@@ -151,7 +155,7 @@ static int deflate_file_to_buffer_gzip(server *srv, connection *con, plugin_data
 	unsigned char *c;
 	unsigned long crc;
 	z_stream z;
-		
+	
 	UNUSED(srv);
 	UNUSED(con);
 
@@ -318,7 +322,7 @@ static int deflate_file_to_buffer_bzip2(server *srv, connection *con, plugin_dat
 }
 #endif
 
-static int deflate_file_to_file(server *srv, connection *con, plugin_data *p, buffer *fn, file_cache_entry *fce, int type) {
+static int deflate_file_to_file(server *srv, connection *con, plugin_data *p, buffer *fn, stat_cache_entry *sce, int type) {
 	int ifd, ofd;
 	int ret = -1;
 	void *start;
@@ -326,14 +330,14 @@ static int deflate_file_to_file(server *srv, connection *con, plugin_data *p, bu
 	ssize_t r;
 	
 	/* overflow */
-	if ((off_t)(fce->st.st_size * 1.1) < fce->st.st_size) return -1;
+	if ((off_t)(sce->st.st_size * 1.1) < sce->st.st_size) return -1;
 	
 	/* don't mmap files > size_t 
 	 * 
 	 * we could use a sliding window, but currently there is no need for it
 	 */
 	
-	if (fce->st.st_size > SIZE_MAX) return -1;
+	if (sce->st.st_size > SIZE_MAX) return -1;
 	
 	buffer_reset(p->ofn);
 	buffer_copy_string_buffer(p->ofn, p->conf.compress_cache_dir);
@@ -353,7 +357,7 @@ static int deflate_file_to_file(server *srv, connection *con, plugin_data *p, bu
 			
 			if (-1 == mkdir(p->b->ptr, 0700)) {
 				if (errno != EEXIST) {
-					log_error_write(srv, __FILE__, __LINE__, "ssss", "creating cache-directory", p->b->ptr, "failed", strerror(errno));
+					log_error_write(srv, __FILE__, __LINE__, "sbss", "creating cache-directory", p->b, "failed", strerror(errno));
 					
 					return -1;
 				}
@@ -380,9 +384,9 @@ static int deflate_file_to_file(server *srv, connection *con, plugin_data *p, bu
 		return -1;
 	}
 	
-	buffer_append_string_buffer(p->ofn, fce->etag);
+	buffer_append_string_buffer(p->ofn, sce->etag);
 	
-	if (-1 == (ofd = open(p->ofn->ptr, O_WRONLY | O_CREAT | O_EXCL, 0600))) {
+	if (-1 == (ofd = open(p->ofn->ptr, O_WRONLY | O_CREAT | O_EXCL | O_BINARY, 0600))) {
 		if (errno == EEXIST) {
 			/* cache-entry exists */
 #if 0
@@ -400,7 +404,7 @@ static int deflate_file_to_file(server *srv, connection *con, plugin_data *p, bu
 #if 0
 	log_error_write(srv, __FILE__, __LINE__, "bs", p->ofn, "compress-cache miss");
 #endif	
-	if (-1 == (ifd = open(filename, O_RDONLY))) {
+	if (-1 == (ifd = open(filename, O_RDONLY | O_BINARY))) {
 		log_error_write(srv, __FILE__, __LINE__, "sbss", "opening plain-file", fn, "failed", strerror(errno));
 		
 		close(ofd);
@@ -409,7 +413,7 @@ static int deflate_file_to_file(server *srv, connection *con, plugin_data *p, bu
 	}
 	
 	
-	if (MAP_FAILED == (start = mmap(NULL, fce->st.st_size, PROT_READ, MAP_SHARED, ifd, 0))) {
+	if (MAP_FAILED == (start = mmap(NULL, sce->st.st_size, PROT_READ, MAP_SHARED, ifd, 0))) {
 		log_error_write(srv, __FILE__, __LINE__, "sbss", "mmaping", fn, "failed", strerror(errno));
 		
 		close(ofd);
@@ -420,15 +424,15 @@ static int deflate_file_to_file(server *srv, connection *con, plugin_data *p, bu
 	switch(type) {
 #ifdef USE_ZLIB
 	case HTTP_ACCEPT_ENCODING_GZIP: 
-		ret = deflate_file_to_buffer_gzip(srv, con, p, start, fce->st.st_size, fce->st.st_mtime);
+		ret = deflate_file_to_buffer_gzip(srv, con, p, start, sce->st.st_size, sce->st.st_mtime);
 		break;
 	case HTTP_ACCEPT_ENCODING_DEFLATE: 
-		ret = deflate_file_to_buffer_deflate(srv, con, p, start, fce->st.st_size);
+		ret = deflate_file_to_buffer_deflate(srv, con, p, start, sce->st.st_size);
 		break;
 #endif
 #ifdef USE_BZ2LIB
 	case HTTP_ACCEPT_ENCODING_BZIP2: 
-		ret = deflate_file_to_buffer_bzip2(srv, con, p, start, fce->st.st_size);
+		ret = deflate_file_to_buffer_bzip2(srv, con, p, start, sce->st.st_size);
 		break;
 #endif
 	default:
@@ -444,7 +448,7 @@ static int deflate_file_to_file(server *srv, connection *con, plugin_data *p, bu
 		
 	}
 		
-	munmap(start, fce->st.st_size);
+	munmap(start, sce->st.st_size);
 	close(ofd);
 	close(ifd);
 	
@@ -455,31 +459,31 @@ static int deflate_file_to_file(server *srv, connection *con, plugin_data *p, bu
 	return 0;
 }
 
-static int deflate_file_to_buffer(server *srv, connection *con, plugin_data *p, buffer *fn, file_cache_entry *fce, int type) {
+static int deflate_file_to_buffer(server *srv, connection *con, plugin_data *p, buffer *fn, stat_cache_entry *sce, int type) {
 	int ifd;
 	int ret = -1;
 	void *start;
 	buffer *b;
 	
 	/* overflow */
-	if ((off_t)(fce->st.st_size * 1.1) < fce->st.st_size) return -1;
+	if ((off_t)(sce->st.st_size * 1.1) < sce->st.st_size) return -1;
 	
 	/* don't mmap files > size_t 
 	 * 
 	 * we could use a sliding window, but currently there is no need for it
 	 */
 	
-	if (fce->st.st_size > SIZE_MAX) return -1;
+	if (sce->st.st_size > SIZE_MAX) return -1;
 	
 	
-	if (-1 == (ifd = open(fn->ptr, O_RDONLY))) {
+	if (-1 == (ifd = open(fn->ptr, O_RDONLY | O_BINARY))) {
 		log_error_write(srv, __FILE__, __LINE__, "sbss", "opening plain-file", fn, "failed", strerror(errno));
 		
 		return -1;
 	}
 	
 	
-	if (MAP_FAILED == (start = mmap(NULL, fce->st.st_size, PROT_READ, MAP_SHARED, ifd, 0))) {
+	if (MAP_FAILED == (start = mmap(NULL, sce->st.st_size, PROT_READ, MAP_SHARED, ifd, 0))) {
 		log_error_write(srv, __FILE__, __LINE__, "sbss", "mmaping", fn, "failed", strerror(errno));
 		
 		close(ifd);
@@ -489,15 +493,15 @@ static int deflate_file_to_buffer(server *srv, connection *con, plugin_data *p,
 	switch(type) {
 #ifdef USE_ZLIB
 	case HTTP_ACCEPT_ENCODING_GZIP: 
-		ret = deflate_file_to_buffer_gzip(srv, con, p, start, fce->st.st_size, fce->st.st_mtime);
+		ret = deflate_file_to_buffer_gzip(srv, con, p, start, sce->st.st_size, sce->st.st_mtime);
 		break;
 	case HTTP_ACCEPT_ENCODING_DEFLATE: 
-		ret = deflate_file_to_buffer_deflate(srv, con, p, start, fce->st.st_size);
+		ret = deflate_file_to_buffer_deflate(srv, con, p, start, sce->st.st_size);
 		break;
 #endif
 #ifdef USE_BZ2LIB
 	case HTTP_ACCEPT_ENCODING_BZIP2: 
-		ret = deflate_file_to_buffer_bzip2(srv, con, p, start, fce->st.st_size);
+		ret = deflate_file_to_buffer_bzip2(srv, con, p, start, sce->st.st_size);
 		break;
 #endif
 	default:
@@ -505,7 +509,7 @@ static int deflate_file_to_buffer(server *srv, connection *con, plugin_data *p,
 		break;
 	}
 		
-	munmap(start, fce->st.st_size);
+	munmap(start, sce->st.st_size);
 	close(ifd);
 	
 	if (ret != 0) return -1;
@@ -525,16 +529,18 @@ static int deflate_file_to_buffer(server *srv, connection *con, plugin_data *p,
 
 #define PATCH(x) \
 	p->conf.x = s->x;
-static int mod_compress_patch_connection(server *srv, connection *con, plugin_data *p, const char *stage, size_t stage_len) {
+static int mod_compress_patch_connection(server *srv, connection *con, plugin_data *p) {
 	size_t i, j;
+	plugin_config *s = p->config_storage[0];
+
+	PATCH(compress_cache_dir);
+	PATCH(compress);
+	PATCH(compress_max_filesize);
 	
 	/* skip the first, the global context */
 	for (i = 1; i < srv->config_context->used; i++) {
 		data_config *dc = (data_config *)srv->config_context->data[i];
-		plugin_config *s = p->config_storage[i];
-		
-		/* not our stage */
-		if (!buffer_is_equal_string(dc->comp_key, stage, stage_len)) continue;
+		s = p->config_storage[i];
 		
 		/* condition didn't match */
 		if (!config_check_cond(srv, con, dc)) continue;
@@ -555,50 +561,32 @@ static int mod_compress_patch_connection(server *srv, connection *con, plugin_da
 	
 	return 0;
 }
-
-static int mod_compress_setup_connection(server *srv, connection *con, plugin_data *p) {
-	plugin_config *s = p->config_storage[0];
-	UNUSED(srv);
-	UNUSED(con);
-		
-	PATCH(compress_cache_dir);
-	PATCH(compress);
-	PATCH(compress_max_filesize);
-	
-	return 0;
-}
 #undef PATCH
 
 PHYSICALPATH_FUNC(mod_compress_physical) {
 	plugin_data *p = p_d;
-	data_string *content_ds;
-	size_t m, i;
+	size_t m;
 	off_t max_fsize;
+	stat_cache_entry *sce = NULL;
 	
 	/* only GET and POST can get compressed */
 	if (con->request.http_method != HTTP_METHOD_GET && 
 	    con->request.http_method != HTTP_METHOD_POST) {
 		return HANDLER_GO_ON;
 	}
-	
-	mod_compress_setup_connection(srv, con, p);
-	for (i = 0; i < srv->config_patches->used; i++) {
-		buffer *patch = srv->config_patches->ptr[i];
-		
-		mod_compress_patch_connection(srv, con, p, CONST_BUF_LEN(patch));
-	}
-	
-	
-	max_fsize = p->conf.compress_max_filesize;
-	
-	/* don't compress files that are too large as we need to much time to handle them */
-	if (max_fsize && (con->fce->st.st_size >> 10) > max_fsize) return HANDLER_GO_ON;
-	
-	if (NULL == (content_ds = (data_string *)array_get_element(con->response.headers, "Content-Type"))) {
-		log_error_write(srv, __FILE__, __LINE__, "sbb", "Content-Type is not set for", con->physical.path, con->uri.path);
-		
+
+	if (buffer_is_empty(con->physical.path)) {
 		return HANDLER_GO_ON;
 	}
+	
+	mod_compress_patch_connection(srv, con, p);
+	
+	max_fsize = p->conf.compress_max_filesize;
+
+	stat_cache_get_entry(srv, con, con->physical.path, &sce);
+
+	/* don't compress files that are too large as we need to much time to handle them */
+	if (max_fsize && (sce->st.st_size >> 10) > max_fsize) return HANDLER_GO_ON;
 		
 	/* check if mimetype is in compress-config */
 	for (m = 0; m < p->conf.compress->used; m++) {
@@ -610,7 +598,7 @@ PHYSICALPATH_FUNC(mod_compress_physical) {
 			return HANDLER_GO_ON;
 		}
 		
-		if (buffer_is_equal(compress_ds->value, content_ds->value)) {
+		if (buffer_is_equal(compress_ds->value, sce->content_type)) {
 			/* mimetype found */
 			data_string *ds;
 				
@@ -665,55 +653,28 @@ PHYSICALPATH_FUNC(mod_compress_physical) {
 					/* deflate it */
 					if (p->conf.compress_cache_dir->used) {
 						if (0 == deflate_file_to_file(srv, con, p,
-									      con->physical.path, con->fce, compression_type)) {
-							struct tm *tm;
-							time_t last_mod;
+									      con->physical.path, sce, compression_type)) {
+							buffer *mtime;
 							
-							response_header_insert(srv, con, CONST_STR_LEN("Content-Encoding"), compression_name, strlen(compression_name));
+							response_header_overwrite(srv, con, CONST_STR_LEN("Content-Encoding"), compression_name, strlen(compression_name));
 							
-							/* Set Last-Modified of ORIGINAL file */
-							last_mod = con->fce->st.st_mtime;
+							mtime = strftime_cache_get(srv, sce->st.st_mtime);
+							response_header_overwrite(srv, con, CONST_STR_LEN("Last-Modified"), CONST_BUF_LEN(mtime));
+
+							etag_mutate(con->physical.etag, sce->etag);
+							response_header_overwrite(srv, con, CONST_STR_LEN("ETag"), CONST_BUF_LEN(con->physical.etag));
+
+							response_header_overwrite(srv, con, CONST_STR_LEN("Content-Type"), CONST_BUF_LEN(sce->content_type));
 							
-							for (i = 0; i < FILE_CACHE_MAX; i++) {
-								if (srv->mtime_cache[i].mtime == last_mod) break;
-								
-								if (srv->mtime_cache[i].mtime == 0) {
-									srv->mtime_cache[i].mtime = last_mod;
-									
-									buffer_prepare_copy(srv->mtime_cache[i].str, 1024);
-									
-									tm = gmtime(&(srv->mtime_cache[i].mtime));
-									srv->mtime_cache[i].str->used = strftime(srv->mtime_cache[i].str->ptr, 
-														 srv->mtime_cache[i].str->size - 1,
-														 "%a, %d %b %Y %H:%M:%S GMT", tm);
-								
-									srv->mtime_cache[i].str->used++;
-									break;
-								}
-							}
-							
-							if (i == FILE_CACHE_MAX) {
-								i = 0;
-								
-								srv->mtime_cache[i].mtime = last_mod;
-								buffer_prepare_copy(srv->mtime_cache[i].str, 1024);
-								tm = gmtime(&(srv->mtime_cache[i].mtime));
-								srv->mtime_cache[i].str->used = strftime(srv->mtime_cache[i].str->ptr, 
-													 srv->mtime_cache[i].str->size - 1,
-													 "%a, %d %b %Y %H:%M:%S GMT", tm);
-								srv->mtime_cache[i].str->used++;
-							}
-							
-							response_header_insert(srv, con, CONST_STR_LEN("Last-Modified"), CONST_BUF_LEN(srv->mtime_cache[i].str));
-							
-							return HANDLER_FINISHED;
+							return HANDLER_GO_ON;
 						}
 					} else if (0 == deflate_file_to_buffer(srv, con, p,
-									       con->physical.path, con->fce, compression_type)) {
+									       con->physical.path, sce, compression_type)) {
 							
-						response_header_insert(srv, con, CONST_STR_LEN("Content-Encoding"), compression_name, strlen(compression_name));
+						response_header_overwrite(srv, con, CONST_STR_LEN("Content-Encoding"), compression_name, strlen(compression_name));
+						response_header_overwrite(srv, con, CONST_STR_LEN("Content-Type"), CONST_BUF_LEN(sce->content_type));
 						
-						return HANDLER_FINISHED;
+						return HANDLER_GO_ON;
 					}
 					break;
 				}
@@ -730,7 +691,7 @@ int mod_compress_plugin_init(plugin *p) {
 	
 	p->init        = mod_compress_init;
 	p->set_defaults = mod_compress_setdefaults;
-	p->handle_physical_path  = mod_compress_physical;
+	p->handle_subrequest_start  = mod_compress_physical;
 	p->cleanup     = mod_compress_free;
 	
 	p->data        = NULL;
