@@ -22,6 +22,7 @@
 #include "plugin.h"
 
 #include "inet_ntop_cache.h"
+#include "stat_cache.h"
 
 #include <fastcgi.h>
 #include <stdio.h>
@@ -216,6 +217,13 @@ typedef struct {
 	 */
 	
 	unsigned short break_scriptfilename_for_php;
+
+	/*
+	 * If the backend includes X-LIGHTTPD-send-file in the response
+	 * we use the value as filename and ignore the content.
+	 *
+	 */
+	unsigned short allow_xsendfile;
 		
 	ssize_t load; /* replace by host->load */
 
@@ -302,12 +310,6 @@ typedef enum { FCGI_STATE_INIT, FCGI_STATE_CONNECT, FCGI_STATE_PREPARE_WRITE,
 } fcgi_connection_state_t;
 
 typedef struct {
-	buffer  *response; 
-	size_t   response_len;
-	int      response_type;
-	int      response_padding;
-	size_t   response_request_id;
-	
 	fcgi_proc *proc;
 	fcgi_extension_host *host;
 	
@@ -319,7 +321,7 @@ typedef struct {
 	buffer   *write_buffer;
 	size_t    write_offset;
 	
-	read_buffer *rb;
+	chunkqueue *rb;
 	
 	buffer   *response_header;
 	
@@ -331,6 +333,8 @@ typedef struct {
 
 	pid_t     pid;
 	int       got_proc;
+
+	int       send_content_body;
 	
 	plugin_config conf;
 	
@@ -354,7 +358,6 @@ static handler_ctx * handler_ctx_init() {
 	
 	hctx->fde_ndx = -1;
 	
-	hctx->response = buffer_init();
 	hctx->response_header = buffer_init();
 	hctx->write_buffer = buffer_init();
 	
@@ -362,27 +365,22 @@ static handler_ctx * handler_ctx_init() {
 	hctx->state = FCGI_STATE_INIT;
 	hctx->proc = NULL;
 	
-	hctx->response_len = 0;
-	hctx->response_type = 0;
-	hctx->response_padding = 0;
-	hctx->response_request_id = 0;
 	hctx->fd = -1;
 	
 	hctx->reconnects = 0;
+	hctx->send_content_body = 1;
+
+	hctx->rb = chunkqueue_init();
 	
 	return hctx;
 }
 
 static void handler_ctx_free(handler_ctx *hctx) {
-	buffer_free(hctx->response);
 	buffer_free(hctx->response_header);
 	buffer_free(hctx->write_buffer);
-	
-	if (hctx->rb) {
-		if (hctx->rb->ptr) free(hctx->rb->ptr);
-		free(hctx->rb);
-	}
-	
+
+	chunkqueue_free(hctx->rb);
+
 	free(hctx);
 }
 
@@ -622,7 +620,7 @@ static int env_add(char_array *env, const char *key, size_t key_len, const char
 	if (env->size == 0) {
 		env->size = 16;
 		env->ptr = malloc(env->size * sizeof(*env->ptr));
-	} else if (env->size == env->used) {
+	} else if (env->size == env->used + 1) {
 		env->size += 16;
 		env->ptr = realloc(env->ptr, env->size * sizeof(*env->ptr));
 	}
@@ -632,6 +630,62 @@ static int env_add(char_array *env, const char *key, size_t key_len, const char
 	return 0;
 }
 
+static int parse_binpath(char_array *env, buffer *b) {
+	char *start;
+	size_t i;
+	/* search for spaces */
+
+	start = b->ptr;
+	for (i = 0; i < b->used - 1; i++) {
+		switch(b->ptr[i]) {
+		case ' ':
+		case '\t':
+			/* a WS, stop here and copy the argument */
+
+			if (env->size == 0) {
+				env->size = 16;
+				env->ptr = malloc(env->size * sizeof(*env->ptr));
+			} else if (env->size == env->used) { 
+				env->size += 16;
+				env->ptr = realloc(env->ptr, env->size * sizeof(*env->ptr));
+			}
+			
+			b->ptr[i] = '\0';
+
+			env->ptr[env->used++] = start;
+			
+			start = b->ptr + i + 1;
+			break;
+		default:
+			break;
+		}
+	}
+
+	if (env->size == 0) {
+		env->size = 16;
+		env->ptr = malloc(env->size * sizeof(*env->ptr));
+	} else if (env->size == env->used) { /* we need one extra for the terminating NULL */
+		env->size += 16;
+		env->ptr = realloc(env->ptr, env->size * sizeof(*env->ptr));
+	}
+
+	/* the rest */
+	env->ptr[env->used++] = start;
+
+	if (env->size == 0) {
+		env->size = 16;
+		env->ptr = malloc(env->size * sizeof(*env->ptr));
+	} else if (env->size == env->used) { /* we need one extra for the terminating NULL */
+		env->size += 16;
+		env->ptr = realloc(env->ptr, env->size * sizeof(*env->ptr));
+	}
+
+	/* terminate */
+	env->ptr[env->used++] = NULL;
+
+	return 0;
+}
+
 static int fcgi_spawn_connection(server *srv, 
 				 plugin_data *p,
 				 fcgi_extension_host *host,
@@ -690,8 +744,8 @@ static int fcgi_spawn_connection(server *srv,
 			
 			if (NULL == (he = gethostbyname(host->host->ptr))) {
 				log_error_write(srv, __FILE__, __LINE__, 
-						"ssb", "gethostbyname failed: ", 
-						hstrerror(h_errno), host->host);
+						"sdb", "gethostbyname failed: ", 
+						h_errno, host->host);
 				return -1;
 			}
 			
@@ -765,16 +819,19 @@ static int fcgi_spawn_connection(server *srv,
 #ifdef HAVE_FORK	
 		switch ((child = fork())) {
 		case 0: {
-			buffer *b;
 			size_t i = 0;
 			char_array env;
-			
+			char_array arg;
 			
 			/* create environment */
 			env.ptr = NULL;
 			env.size = 0;
 			env.used = 0;
 			
+			arg.ptr = NULL;
+			arg.size = 0;
+			arg.used = 0;
+
 			if(fcgi_fd != FCGI_LISTENSOCK_FILENO) {
 				close(FCGI_LISTENSOCK_FILENO);
 				dup2(fcgi_fd, FCGI_LISTENSOCK_FILENO);
@@ -824,16 +881,14 @@ static int fcgi_spawn_connection(server *srv,
 			}
 			
 			env.ptr[env.used] = NULL;
-			
-			b = buffer_init();
-			buffer_copy_string(b, "exec ");
-			buffer_append_string_buffer(b, host->bin_path);
+
+			parse_binpath(&arg, host->bin_path);
 			
 			/* exec the cgi */
-			execle("/bin/sh", "sh", "-c", b->ptr, NULL, env.ptr);
+			execve(arg.ptr[0], arg.ptr, env.ptr);
 			
 			log_error_write(srv, __FILE__, __LINE__, "sbs", 
-					"execl failed for:", host->bin_path, strerror(errno));
+					"execve failed for:", host->bin_path, strerror(errno));
 			
 			exit(errno);
 			
@@ -861,8 +916,13 @@ static int fcgi_spawn_connection(server *srv,
 				/* the child should not terminate at all */
 				if (WIFEXITED(status)) {
 					log_error_write(srv, __FILE__, __LINE__, "sd", 
-							"child exited (is this a FastCGI binary ?):", 
+							"child exited:", 
 							WEXITSTATUS(status));
+					log_error_write(srv, __FILE__, __LINE__, "s", 
+							"if you try do run PHP as FastCGI backend make sure you use the FastCGI enabled version.\n"
+							"You can find out if it is the right one by executing 'php -v' and it should display '(cgi-fcgi)' "
+							"in the output, NOT (cgi) NOR (cli)\n"
+							"For more information check http://www.lighttpd.net/documentation/fastcgi.html#preparing-php-as-a-fastcgi-program");
 				} else if (WIFSIGNALED(status)) {
 					log_error_write(srv, __FILE__, __LINE__, "sd", 
 							"child signaled:", 
@@ -1004,6 +1064,7 @@ SETDEFAULTS_FUNC(mod_fastcgi_set_defaults) {
 						{ "bin-copy-environment", NULL, T_CONFIG_ARRAY, T_CONFIG_SCOPE_CONNECTION },     /* 13 */
 						
 						{ "broken-scriptfilename", NULL, T_CONFIG_BOOLEAN, T_CONFIG_SCOPE_CONNECTION },  /* 14 */
+						{ "allow-x-send-file", NULL, T_CONFIG_BOOLEAN, T_CONFIG_SCOPE_CONNECTION },  /* 15 */
 						
 						{ NULL,                NULL, T_CONFIG_UNSET, T_CONFIG_SCOPE_UNSET }
 					};
@@ -1027,6 +1088,7 @@ SETDEFAULTS_FUNC(mod_fastcgi_set_defaults) {
 					df->mode = FCGI_RESPONDER;
 					df->disable_time = 60;
 					df->break_scriptfilename_for_php = 0;
+					df->allow_xsendfile = 0; /* handle X-LIGHTTPD-send-file */
 					
 					fcv[0].destination = df->host;
 					fcv[1].destination = df->docroot;
@@ -1045,6 +1107,7 @@ SETDEFAULTS_FUNC(mod_fastcgi_set_defaults) {
 					fcv[12].destination = df->bin_env;
 					fcv[13].destination = df->bin_env_copy;
 					fcv[14].destination = &(df->break_scriptfilename_for_php);
+					fcv[15].destination = &(df->allow_xsendfile);
 					
 					
 					if (0 != config_insert_values_internal(srv, da_host->value, fcv)) {
@@ -1249,7 +1312,7 @@ static int fcgi_requestid_del(server *srv, plugin_data *p, size_t request_id) {
 	return 0;
 }
 
-void fcgi_connection_cleanup(server *srv, handler_ctx *hctx) {
+void fcgi_connection_close(server *srv, handler_ctx *hctx) {
 	plugin_data *p;
 	connection  *con;
 	
@@ -1348,7 +1411,7 @@ static int fcgi_reconnect(server *srv, handler_ctx *hctx) {
 static handler_t fcgi_connection_reset(server *srv, connection *con, void *p_d) {
 	plugin_data *p = p_d;
 	
-	fcgi_connection_cleanup(srv, con->plugin_ctx[p->id]);
+	fcgi_connection_close(srv, con->plugin_ctx[p->id]);
 	
 	return HANDLER_GO_ON;
 }
@@ -1473,11 +1536,10 @@ static int fcgi_establish_connection(server *srv, handler_ctx *hctx) {
 					proc->port, proc->socket);
 
 			if (errno == EAGAIN) {
-				/* this is Linux only */
-				
-				log_error_write(srv, __FILE__, __LINE__, "s", 
-						"If this happend on Linux: You have been run out of local ports. "
-						"Check the manual, section Performance how to handle this.");
+				log_error_write(srv, __FILE__, __LINE__, "sd", 
+						"This means that the you have more incoming requests than your fastcgi-backend can handle in parallel. "
+						"Perhaps it helps to spawn more fastcgi backend or php-children, if not decrease server.max-connections."
+						"The load for this fastcgi backend is:", proc->load);
 			} 
 			
 			return -1;
@@ -1512,9 +1574,15 @@ static int fcgi_env_add_request_headers(server *srv, connection *con, plugin_dat
 			
 			buffer_prepare_append(srv->tmp_buf, ds->key->used + 2);
 			for (j = 0; j < ds->key->used - 1; j++) {
-				srv->tmp_buf->ptr[srv->tmp_buf->used++] = 
-					light_isalpha(ds->key->ptr[j]) ? 
-					ds->key->ptr[j] & ~32 : '_';
+				char c = '_';
+				if (light_isalpha(ds->key->ptr[j])) {
+					/* upper-case */
+					c = ds->key->ptr[j] & ~32;
+				} else if (light_isdigit(ds->key->ptr[j])) {
+					/* copy */
+					c = ds->key->ptr[j];
+				}
+				srv->tmp_buf->ptr[srv->tmp_buf->used++] = c;
 			}
 			srv->tmp_buf->ptr[srv->tmp_buf->used++] = '\0';
 			
@@ -1872,11 +1940,103 @@ static int fcgi_response_parse(server *srv, connection *con, plugin_data *p, buf
 	return 0;
 }
 
+typedef struct {
+	buffer  *b; 
+	size_t   len;
+	int      type;
+	int      padding;
+	size_t   request_id;
+} fastcgi_response_packet;
+
+static int fastcgi_get_packet(server *srv, handler_ctx *hctx, fastcgi_response_packet *packet) {
+	chunk *	c;
+	size_t offset = 0;
+	size_t toread = 0;
+	FCGI_Header *header;
+
+	if (!hctx->rb->first) return -1;
+
+	packet->b = buffer_init();
+	packet->len = 0;
+	packet->type = 0;
+	packet->padding = 0;
+	packet->request_id = 0;
+
+	/* get at least the FastCGI header */
+	for (c = hctx->rb->first; c; c = c->next) {
+		if (packet->b->used == 0) {
+			buffer_copy_string_len(packet->b, c->mem->ptr + c->offset, c->mem->used - c->offset - 1);
+		} else {
+			buffer_append_string_len(packet->b, c->mem->ptr + c->offset, c->mem->used - c->offset - 1);
+		}
+
+		if (packet->b->used >= sizeof(*header) + 1) break;
+	}
+
+	if ((packet->b->used == 0) ||
+	    (packet->b->used - 1 < sizeof(FCGI_Header))) {
+		/* no header */
+		buffer_free(packet->b);
+
+		log_error_write(srv, __FILE__, __LINE__, "s", "FastCGI: header to small");
+		return -1;
+	}
+
+	/* we have at least a header, now check how much me have to fetch */ 
+	header = (FCGI_Header *)(packet->b->ptr);
+			
+	packet->len = (header->contentLengthB0 | (header->contentLengthB1 << 8)) + header->paddingLength;
+	packet->request_id = (header->requestIdB0 | (header->requestIdB1 << 8));
+	packet->type = header->type;
+	packet->padding = header->paddingLength;
+
+	/* the first bytes in packet->b are the header */
+	offset = sizeof(*header);
+
+	/* ->b should only be the content */
+	buffer_reset(packet->b);
+
+	if (packet->len) {
+		/* copy the content */
+		for (; c && (packet->b->used < packet->len + 1); c = c->next) {
+			toread = c->mem->used - c->offset - offset - 1 > packet->len ? packet->len : c->mem->used - c->offset - offset - 1;
+			
+			buffer_append_string_len(packet->b, c->mem->ptr + c->offset + offset, toread);
+			offset = 0;
+		}
+
+		if (packet->b->used < packet->len + 1) {
+			/* we didn't got the full packet */
+
+			buffer_free(packet->b);
+			return -1;
+		}
+
+		packet->b->used -= packet->padding;
+		packet->b->ptr[packet->b->used - 1] = '\0';
+	}
+
+	/* tag the chunks as read */
+	toread = packet->len + sizeof(FCGI_Header);
+	for (c = hctx->rb->first; c && toread; c = c->next) {
+		if (c->mem->used - c->offset - 1 <= toread) {
+			/* we read this whole buffer, move it to unused */
+			toread -= c->mem->used - c->offset - 1;
+			c->offset = c->mem->used - 1; /* everthing has been written */
+		} else {
+			c->offset += toread;
+			toread = 0;
+		}
+	}
+
+	chunkqueue_remove_finished_chunks(hctx->rb);
+	
+	return 0;
+}
 
 static int fcgi_demux_response(server *srv, handler_ctx *hctx) {
-	ssize_t len;
 	int fin = 0;
-	int b;
+	int toread;
 	ssize_t r;
 	
 	plugin_data *p    = hctx->plugin_data;
@@ -1888,7 +2048,7 @@ static int fcgi_demux_response(server *srv, handler_ctx *hctx) {
 	/* 
 	 * check how much we have to read 
 	 */
-	if (ioctl(hctx->fd, FIONREAD, &b)) {
+	if (ioctl(hctx->fd, FIONREAD, &toread)) {
 		log_error_write(srv, __FILE__, __LINE__, "sd", 
 				"unexpected end-of-file (perhaps the fastcgi process died):",
 				fcgi_fd);
@@ -1896,21 +2056,15 @@ static int fcgi_demux_response(server *srv, handler_ctx *hctx) {
 	}
 	
 	/* init read-buffer */
-	if (hctx->rb == NULL) {
-		hctx->rb = calloc(1, sizeof(*hctx->rb));
-	}
 	
-	if (b > 0) {
-		if (hctx->rb->size == 0) {
-			hctx->rb->size = b;
-			hctx->rb->ptr = malloc(hctx->rb->size * sizeof(*hctx->rb->ptr));
-		} else if (hctx->rb->size < hctx->rb->used + b) {
-			hctx->rb->size += b;
-			hctx->rb->ptr = realloc(hctx->rb->ptr, hctx->rb->size * sizeof(*hctx->rb->ptr));
-		}
-		
+	if (toread > 0) {
+		buffer *b;
+
+		b = chunkqueue_get_append_buffer(hctx->rb);
+		buffer_prepare_copy(b, toread + 1);
+
 		/* append to read-buffer */
-		if (-1 == (r = read(hctx->fd, hctx->rb->ptr + hctx->rb->used, b))) {
+		if (-1 == (r = read(hctx->fd, b->ptr, toread))) {
 			log_error_write(srv, __FILE__, __LINE__, "sds", 
 					"unexpected end-of-file (perhaps the fastcgi process died):",
 					fcgi_fd, strerror(errno));
@@ -1919,8 +2073,9 @@ static int fcgi_demux_response(server *srv, handler_ctx *hctx) {
 		
 		/* this should be catched by the b > 0 above */
 		assert(r);
-		
-		hctx->rb->used += r;
+
+		b->used = r + 1; /* one extra for the fake \0 */
+		b->ptr[b->used - 1] = '\0';
 	} else {
 		log_error_write(srv, __FILE__, __LINE__, "ssdsdsd", 
 				"unexpected end-of-file (perhaps the fastcgi process died):",
@@ -1930,183 +2085,110 @@ static int fcgi_demux_response(server *srv, handler_ctx *hctx) {
 		
 		return -1;
 	}
-	
-	/* parse all fcgi packets 
-	 * 
-	 *   start: hctx->rb->ptr 
-	 *   end  : hctx->rb->ptr + hctx->rb->used
-	 * 
-	 */
-	while (fin == 0) {
-		size_t request_id;
-		
-		if (hctx->response_len == 0) {
-			FCGI_Header *header;
-			
-			if (hctx->rb->used - hctx->rb->offset < sizeof(*header)) {
-				/* didn't get the full header packet (most often 0),
-				 * but didn't recieved the final packet either
-				 * 
-				 * we will come back later and finish everything
-				 * 
-				 */
-				
-				hctx->delayed = 1;
-#if 0
-				log_error_write(srv, __FILE__, __LINE__, "sddd", "didn't get the full header: ",
-						hctx->rb->used - hctx->rb->offset, sizeof(*header),
-						fcgi_fd
-						);
-#endif
-				break;
-			}
-#if 0
-			fprintf(stderr, "fcgi-version: %02x\n", hctx->rb->ptr[hctx->rb->offset]);
-#endif
-			
-			header = (FCGI_Header *)(hctx->rb->ptr + hctx->rb->offset);
-			hctx->rb->offset += sizeof(*header);
-			
-			len = (header->contentLengthB0 | (header->contentLengthB1 << 8)) + header->paddingLength;
-			request_id = (header->requestIdB0 | (header->requestIdB1 << 8));
 
-			hctx->response_len = len;
-			hctx->response_request_id = request_id;
-			hctx->response_type = header->type;
-			hctx->response_padding = header->paddingLength;
-			
-#if 0
-			log_error_write(srv, __FILE__, __LINE__, "sddd", "offset: ",
-					fcgi_fd, hctx->rb->offset, header->type
-					);
-#endif
-			
-		} else {
-			len = hctx->response_len;
-		}
-		
-		if (hctx->rb->used - hctx->rb->offset < hctx->response_len) {
-			/* we are not finished yet */
+	/*
+	 * parse the fastcgi packets and forward the content to the write-queue
+	 *
+	 */	
+	while (fin == 0) {
+		fastcgi_response_packet packet;
+
+		/* check if we have at least one packet */
+		if (0 != fastcgi_get_packet(srv, hctx, &packet)) {
+			/* no full packet */
+
+			hctx->delayed = 1;
+
 			break;
 		}
-		
-		hctx->response->ptr = hctx->rb->ptr + hctx->rb->offset;
-		hctx->rb->offset += hctx->response_len;
-#if 0
-		log_error_write(srv, __FILE__, __LINE__, "sdd", "offset: ",
-				fcgi_fd, hctx->rb->offset
-				);
-#endif
-		
-		/* remove padding */
-#if 0
-		hctx->response->ptr[hctx->response_len - hctx->response_padding] = '\0';
-#endif
-		hctx->response->used = hctx->response_len - hctx->response_padding + 1;
-		
-		/* mark the fast-cgi packet as finished */
-		hctx->response_len = 0;
-		
-		switch(hctx->response_type) {
-		case FCGI_STDOUT:
-			if (len) {
-#if 0
-				log_error_write(srv, __FILE__, __LINE__, "sdb", "len", len, hctx->response);
-#endif
-				
-				if (0 == con->got_response) {
-					con->got_response = 1;
-					buffer_prepare_copy(hctx->response_header, 128);
-				}
-				
-				if (0 == con->file_started) {
-					char *c;
-					size_t hlen;
-					
-					/* search for header terminator 
-					 * 
-					 * if we start with \r\n check if last packet terminated with \r\n
-					 * if we start with \n check if last packet terminated with \n
-					 * search for \r\n\r\n
-					 * search for \n\n
-					 */
-					
-					if (hctx->response->used > 2 &&
-					    hctx->response->ptr[0] == '\r' &&
-					    hctx->response->ptr[1] == '\n' &&
-					    hctx->response_header->used > 3 &&
-					    hctx->response_header->ptr[hctx->response_header->used - 3] == '\r' &&
-					    hctx->response_header->ptr[hctx->response_header->used - 2] == '\n') {
-						hlen = 2;
-						c = hctx->response->ptr + 2;
-					} else if (hctx->response->used > 2 &&
-						   hctx->response->ptr[0] == '\n' &&
-						   hctx->response_header->used > 2 &&
-						   hctx->response_header->ptr[hctx->response_header->used - 2] == '\n') {
-						hlen = 1;
-						c = hctx->response->ptr + 1;
-					} else if (NULL != (c = buffer_search_string_len(hctx->response, "\r\n\r\n", 4))) {
-						c += 4;
-						hlen = c - hctx->response->ptr;
-						
-					} else if (NULL != (c = buffer_search_string_len(hctx->response, "\n\n", 2))) {
-						c += 2;
-						hlen = c - hctx->response->ptr;
-					}
-					
-					if (c != NULL) {
-						size_t blen = hctx->response->used - hlen - 1;
-						/* found */
-						
-						buffer_append_string_len(hctx->response_header, hctx->response->ptr, hlen);
-#if 0
-						log_error_write(srv, __FILE__, __LINE__, "ss", "Header:", hctx->response_header->ptr);
-#endif
-						/* parse the response header */
-						fcgi_response_parse(srv, con, p, hctx->response_header);
-						
-						if (host->mode != FCGI_AUTHORIZER ||
-						    !(con->http_status == 0 ||
-						      con->http_status == 200)) {
-							/* enable chunked-transfer-encoding */
-							if (con->request.http_version == HTTP_VERSION_1_1 &&
-							    !(con->parsed_response & HTTP_CONTENT_LENGTH)) {
-								con->response.transfer_encoding = HTTP_TRANSFER_ENCODING_CHUNKED;
-							}
 
-							con->file_started = 1;
-						
-							if (blen) {
-								http_chunk_append_mem(srv, con, c, blen + 1);
-								joblist_append(srv, con);
-#if 0
-								log_error_write(srv, __FILE__, __LINE__, "sd", "body-len", blen);
-#endif
-							}
-						}
-					} else {
-						/* copy everything */
-						buffer_append_string_buffer(hctx->response_header, hctx->response);
-					}
+		switch(packet.type) {
+		case FCGI_STDOUT:
+			if (packet.len == 0) break;
+
+			/* is the header already finished */
+			if (0 == con->file_started) {
+				char *c;
+				size_t blen;
+				data_string *ds;
+					
+				/* search for header terminator 
+				 * 
+				 * if we start with \r\n check if last packet terminated with \r\n
+				 * if we start with \n check if last packet terminated with \n
+				 * search for \r\n\r\n
+				 * search for \n\n
+				 */
+
+				if (hctx->response_header->used == 0) {
+					buffer_copy_string_buffer(hctx->response_header, packet.b);
 				} else {
-					if (host->mode != FCGI_AUTHORIZER ||
-					    !(con->http_status == 0 ||
-					      con->http_status == 200)) {
-						http_chunk_append_mem(srv, con, hctx->response->ptr, hctx->response->used);
+					buffer_append_string_buffer(hctx->response_header, packet.b);
+				}
+
+				if (NULL != (c = buffer_search_string_len(hctx->response_header, CONST_STR_LEN("\r\n\r\n")))) {
+					blen = hctx->response_header->used - (c - hctx->response_header->ptr) - 4;
+					hctx->response_header->used = (c - hctx->response_header->ptr) + 3;
+					c += 4; /* point the the start of the response */
+				} else if (NULL != (c = buffer_search_string_len(hctx->response_header, CONST_STR_LEN("\n\n")))) {
+					blen = hctx->response_header->used - (c - hctx->response_header->ptr) - 2;
+					hctx->response_header->used = c - hctx->response_header->ptr + 2;
+					c += 2; /* point the the start of the response */
+				} else {
+					/* no luck, no header found */
+					break;
+				}
+
+				/* parse the response header */
+				fcgi_response_parse(srv, con, p, hctx->response_header);
+
+				con->file_started = 1;
+
+				if (host->mode == FCGI_AUTHORIZER &&
+				    (con->http_status == 0 ||
+				     con->http_status == 200)) {
+					/* a authorizer with approved the static request, ignore the content here */
+					hctx->send_content_body = 0;
+				}
+
+				if (host->allow_xsendfile &&
+				    NULL != (ds = (data_string *) array_get_element(con->response.headers, "X-LIGHTTPD-send-file"))) {
+					stat_cache_entry *sce;
+
+					if (HANDLER_ERROR != stat_cache_get_entry(srv, con, ds->value, &sce)) {
+						/* found */
+
+						http_chunk_append_file(srv, con, ds->value, 0, sce->st.st_size);
+						hctx->send_content_body = 0; /* ignore the content */
 						joblist_append(srv, con);
 					}
-#if 0
-					log_error_write(srv, __FILE__, __LINE__, "sd", "body-len", hctx->response->used);
-#endif
 				}
-			} else {
-				/* finished */
+
+						
+				if (hctx->send_content_body && blen > 1) {						
+					/* enable chunked-transfer-encoding */
+					if (con->request.http_version == HTTP_VERSION_1_1 &&
+					    !(con->parsed_response & HTTP_CONTENT_LENGTH)) {
+						con->response.transfer_encoding = HTTP_TRANSFER_ENCODING_CHUNKED;
+					}
+
+					http_chunk_append_mem(srv, con, c, blen);
+					joblist_append(srv, con);
+				}
+			} else if (hctx->send_content_body && packet.b->used > 1) {
+				if (con->request.http_version == HTTP_VERSION_1_1 &&
+				    !(con->parsed_response & HTTP_CONTENT_LENGTH)) {
+					/* enable chunked-transfer-encoding */
+					con->response.transfer_encoding = HTTP_TRANSFER_ENCODING_CHUNKED;
+				}
+
+				http_chunk_append_mem(srv, con, packet.b->ptr, packet.b->used);
+				joblist_append(srv, con);
 			}
-			
 			break;
 		case FCGI_STDERR:
 			log_error_write(srv, __FILE__, __LINE__, "sb", 
-					"FastCGI-stderr:", hctx->response);
+					"FastCGI-stderr:", packet.b);
 			
 			break;
 		case FCGI_END_REQUEST:
@@ -2124,13 +2206,12 @@ static int fcgi_demux_response(server *srv, handler_ctx *hctx) {
 			break;
 		default:
 			log_error_write(srv, __FILE__, __LINE__, "sd", 
-					"FastCGI: header.type not handled: ", hctx->response_type);
+					"FastCGI: header.type not handled: ", packet.type);
 			break;
 		}
+		buffer_free(packet.b);
 	}
 	
-	hctx->response->ptr = NULL;
-
 	return fin;
 }
 
@@ -2650,7 +2731,7 @@ SUBREQUEST_FUNC(mod_fastcgi_handle_subrequest) {
 			}
 			fcgi_restart_dead_procs(srv, p, host);
 			
-			fcgi_connection_cleanup(srv, hctx);
+			fcgi_connection_close(srv, hctx);
 			
 			buffer_reset(con->physical.path);
 			con->mode = DIRECT;
@@ -2672,7 +2753,7 @@ SUBREQUEST_FUNC(mod_fastcgi_handle_subrequest) {
 			
 			return HANDLER_WAIT_FOR_FD;
 		} else {
-			fcgi_connection_cleanup(srv, hctx);
+			fcgi_connection_close(srv, hctx);
 			
 			buffer_reset(con->physical.path);
 			con->mode = DIRECT;
@@ -2694,30 +2775,6 @@ SUBREQUEST_FUNC(mod_fastcgi_handle_subrequest) {
 	}
 }
 
-static handler_t fcgi_connection_close(server *srv, handler_ctx *hctx) {
-	plugin_data *p;
-	connection  *con;
-	
-	if (NULL == hctx) return HANDLER_GO_ON;
-	
-	p    = hctx->plugin_data;
-	con  = hctx->remote_conn;
-	
-	if (con->mode != p->id) return HANDLER_GO_ON;
-	
-	log_error_write(srv, __FILE__, __LINE__, "ssdsd", 
-			"emergency exit: fastcgi:", 
-			"connection-fd:", con->fd,
-			"fcgi-fd:", hctx->fd);
-	
-	
-	
-	fcgi_connection_cleanup(srv, hctx);
-	
-	return HANDLER_FINISHED;
-}
-
-
 static handler_t fcgi_handle_fdevent(void *s, void *ctx, int revents) {
 	server      *srv  = (server *)s;
 	handler_ctx *hctx = ctx;
@@ -2747,13 +2804,13 @@ static handler_t fcgi_handle_fdevent(void *s, void *ctx, int revents) {
 				
 				buffer_copy_string_buffer(con->physical.path, host->docroot);
 				buffer_append_string_buffer(con->physical.path, con->uri.path);
-				fcgi_connection_cleanup(srv, hctx);
+				fcgi_connection_close(srv, hctx);
 				
 				con->mode = DIRECT;
 				con->file_started = 1; /* fcgi_extension won't touch the request afterwards */
 			} else {
 				/* we are done */
-				fcgi_connection_cleanup(srv, hctx);
+				fcgi_connection_close(srv, hctx);
 			}
 			
 			joblist_append(srv, con);
@@ -2825,7 +2882,7 @@ static handler_t fcgi_handle_fdevent(void *s, void *ctx, int revents) {
 						"connection-fd:", con->fd,
 						"fcgi-fd:", hctx->fd);
 				
-				fcgi_connection_cleanup(srv, hctx);
+				fcgi_connection_close(srv, hctx);
 				
 				connection_set_state(srv, con, CON_STATE_HANDLE_REQUEST);
 				buffer_reset(con->physical.path);
@@ -2833,7 +2890,7 @@ static handler_t fcgi_handle_fdevent(void *s, void *ctx, int revents) {
 				con->mode = DIRECT;
 			} else {
 				/* response might have been already started, kill the connection */
-				fcgi_connection_cleanup(srv, hctx);
+				fcgi_connection_close(srv, hctx);
 				
 				log_error_write(srv, __FILE__, __LINE__, "ssdsd", 
 						"response already sent out, termination connection",
@@ -2917,16 +2974,17 @@ static handler_t fcgi_handle_fdevent(void *s, void *ctx, int revents) {
 }
 #define PATCH(x) \
 	p->conf.x = s->x;
-static int fcgi_patch_connection(server *srv, connection *con, plugin_data *p, const char *stage, size_t stage_len) {
+static int fcgi_patch_connection(server *srv, connection *con, plugin_data *p) {
 	size_t i, j;
+	plugin_config *s = p->config_storage[0];
+	
+	PATCH(exts);
+	PATCH(debug);
 	
 	/* skip the first, the global context */
 	for (i = 1; i < srv->config_context->used; i++) {
 		data_config *dc = (data_config *)srv->config_context->data[i];
-		plugin_config *s = p->config_storage[i];
-		
-		/* not our stage */
-		if (!buffer_is_equal_string(dc->comp_key, stage, stage_len)) continue;
+		s = p->config_storage[i];
 		
 		/* condition didn't match */
 		if (!config_check_cond(srv, con, dc)) continue;
@@ -2945,17 +3003,6 @@ static int fcgi_patch_connection(server *srv, connection *con, plugin_data *p, c
 	
 	return 0;
 }
-
-static int fcgi_setup_connection(server *srv, connection *con, plugin_data *p) {
-	plugin_config *s = p->config_storage[0];
-	UNUSED(srv);
-	UNUSED(con);
-	
-	PATCH(exts);
-	PATCH(debug);
-	
-	return 0;
-}
 #undef PATCH
 
 
@@ -2964,7 +3011,7 @@ static handler_t fcgi_check_extension(server *srv, connection *con, void *p_d, i
 	size_t s_len;
 	int used = -1;
 	int ndx;
-	size_t k, i;
+	size_t k;
 	buffer *fn;
 	fcgi_extension *extension = NULL;
 	
@@ -2979,13 +3026,7 @@ static handler_t fcgi_check_extension(server *srv, connection *con, void *p_d, i
 	
 	s_len = fn->used - 1;
 	
-	/* select the right config */
-	fcgi_setup_connection(srv, con, p);
-	for (i = 0; i < srv->config_patches->used; i++) {
-		buffer *patch = srv->config_patches->ptr[i];
-		
-		fcgi_patch_connection(srv, con, p, CONST_BUF_LEN(patch));
-	}
+	fcgi_patch_connection(srv, con, p);
 	
 	/* check if extension matches */
 	for (k = 0; k < p->conf.exts->used; k++) {
@@ -3117,7 +3158,7 @@ static handler_t fcgi_check_extension(server *srv, connection *con, void *p_d, i
 				log_error_write(srv, __FILE__, __LINE__, "s", "handling it in mod_fastcgi");
 			}
 			
-			return HANDLER_FINISHED;
+			return HANDLER_GO_ON;
 		}
 	} else {
 		/* no handler found */
@@ -3176,7 +3217,9 @@ JOBLIST_FUNC(mod_fastcgi_handle_joblist) {
 static handler_t fcgi_connection_close_callback(server *srv, connection *con, void *p_d) {
 	plugin_data *p = p_d;
 	
-	return fcgi_connection_close(srv, con->plugin_ctx[p->id]);
+	fcgi_connection_close(srv, con->plugin_ctx[p->id]);
+
+	return HANDLER_GO_ON;
 }
 
 TRIGGER_FUNC(mod_fastcgi_handle_trigger) {
