@@ -8,9 +8,9 @@
 #include "config.h"
 #include "log.h"
 
-#include "file_cache.h"
+#include "stat_cache.h"
 #ifdef HAVE_MYSQL
-#include <mysql/mysql.h>
+#include <mysql.h>
 #endif
 #ifdef DEBUG_MOD_MYSQL_VHOST
 #define DEBUG
@@ -178,14 +178,14 @@ SERVER_FUNC(mod_mysql_vhost_set_defaults) {
                 { NULL,			NULL, T_CONFIG_UNSET,	T_CONFIG_SCOPE_UNSET }
         };
 	
-	p->config_storage = malloc(srv->config_context->used * sizeof(specific_config *));
+	p->config_storage = calloc(1, srv->config_context->used * sizeof(specific_config *));
 	
 	for (i = 0; i < srv->config_context->used; i++) {
 		plugin_config *s;
 		buffer *sel;
 		
 		
-		s = malloc(sizeof(plugin_config));
+		s = calloc(1, sizeof(plugin_config));
 		s->mydb = buffer_init();
 		s->myuser = buffer_init();
 		s->mypass = buffer_init();
@@ -271,16 +271,20 @@ SERVER_FUNC(mod_mysql_vhost_set_defaults) {
 
 #define PATCH(x) \
 	p->conf.x = s->x;
-static int mod_mysql_vhost_patch_connection(server *srv, connection *con, plugin_data *p, const char *stage, size_t stage_len) {
+static int mod_mysql_vhost_patch_connection(server *srv, connection *con, plugin_data *p) {
 	size_t i, j;
+	plugin_config *s = p->config_storage[0];
+	
+	PATCH(mysql_pre);
+	PATCH(mysql_post);
+#ifdef HAVE_MYSQL
+	PATCH(mysql);
+#endif
 	
 	/* skip the first, the global context */
 	for (i = 1; i < srv->config_context->used; i++) {
 		data_config *dc = (data_config *)srv->config_context->data[i];
-		plugin_config *s = p->config_storage[i];
-		
-		/* not our stage */
-		if (!buffer_is_equal_string(dc->comp_key, stage, stage_len)) continue;
+		s = p->config_storage[i];
 		
 		/* condition didn't match */
 		if (!config_check_cond(srv, con, dc)) continue;
@@ -302,19 +306,6 @@ static int mod_mysql_vhost_patch_connection(server *srv, connection *con, plugin
 	
 	return 0;
 }
-
-static int mod_mysql_vhost_setup_connection(server *srv, connection *con, plugin_data *p) {
-	plugin_config *s = p->config_storage[0];
-	
-	UNUSED(srv);
-	UNUSED(con);
-		
-	PATCH(mysql_pre);
-	PATCH(mysql_post);
-	PATCH(mysql);
-	
-	return 0;
-}
 #undef PATCH
 
 
@@ -322,6 +313,7 @@ static int mod_mysql_vhost_setup_connection(server *srv, connection *con, plugin
 CONNECTION_FUNC(mod_mysql_vhost_handle_docroot) {
 	plugin_data *p = p_d;
 	plugin_connection_data *c;
+	stat_cache_entry *sce;
 
 	unsigned  cols;
 	MYSQL_ROW row;
@@ -331,13 +323,7 @@ CONNECTION_FUNC(mod_mysql_vhost_handle_docroot) {
 	/* no host specified? */
 	if (!con->uri.authority->used) return HANDLER_GO_ON;
 	
-	/* apply conditionals */
-	mod_mysql_vhost_setup_connection(srv, con, p);
-	for (i = 0; i < srv->config_patches->used; i++) {
-		buffer *patch = srv->config_patches->ptr[i];
-		
-		mod_mysql_vhost_patch_connection(srv, con, p, CONST_BUF_LEN(patch));
-	}
+	mod_mysql_vhost_patch_connection(srv, con, p);
 
 	/* sets up connection data if not done yet */
 	c = mod_mysql_vhost_connection_data(srv, con, p_d);
@@ -368,11 +354,12 @@ CONNECTION_FUNC(mod_mysql_vhost_handle_docroot) {
 	/* sanity check that really is a directory */
 	buffer_copy_string(p->tmp_buf, row[0]);
 	BUFFER_APPEND_SLASH(p->tmp_buf);
-	if (file_cache_get_entry(srv, con, p->tmp_buf, &(con->fce)) != HANDLER_GO_ON) {
+
+	if (HANDLER_ERROR == stat_cache_get_entry(srv, con, p->tmp_buf, &sce)) {
 		log_error_write(srv, __FILE__, __LINE__, "sb", strerror(errno), p->tmp_buf);
 		goto ERR500;
 	}
-        if (!S_ISDIR(con->fce->st.st_mode)) {
+        if (!S_ISDIR(sce->st.st_mode)) {
 		log_error_write(srv, __FILE__, __LINE__, "sb", "Not a directory", p->tmp_buf);
 		goto ERR500;
 	}
