@@ -335,21 +335,19 @@ SETDEFAULTS_FUNC(mod_proxy_set_defaults) {
 	return HANDLER_GO_ON;
 }
 
-void proxy_connection_cleanup(server *srv, handler_ctx *hctx) {
+void proxy_connection_close(server *srv, handler_ctx *hctx) {
 	plugin_data *p;
-	connection  *con;
+	connection *con;
 	
 	if (NULL == hctx) return;
 	
 	p    = hctx->plugin_data;
 	con  = hctx->remote_conn;
 	
-	if (con->mode != p->id) return;
-	
-	fdevent_event_del(srv->ev, &(hctx->fde_ndx), hctx->fd);
-	fdevent_unregister(srv->ev, hctx->fd);
-
 	if (hctx->fd != -1) {
+		fdevent_event_del(srv->ev, &(hctx->fde_ndx), hctx->fd);
+		fdevent_unregister(srv->ev, hctx->fd);
+
 		close(hctx->fd);
 		srv->cur_fds--;
 	}
@@ -358,14 +356,6 @@ void proxy_connection_cleanup(server *srv, handler_ctx *hctx) {
 	con->plugin_ctx[p->id] = NULL;	
 }
 
-static handler_t mod_proxy_connection_reset(server *srv, connection *con, void *p_d) {
-	plugin_data *p = p_d;
-	
-	proxy_connection_cleanup(srv, con->plugin_ctx[p->id]);
-	
-	return HANDLER_GO_ON;
-}
-
 static int proxy_establish_connection(server *srv, handler_ctx *hctx) {
 	struct sockaddr *proxy_addr;
 	struct sockaddr_in proxy_addr_in;
@@ -419,19 +409,8 @@ static int proxy_create_env(server *srv, handler_ctx *hctx) {
 	buffer_reset(hctx->write_buffer);
 	
 	/* request line */
-	switch(con->request.http_method) {
-	case HTTP_METHOD_GET:
-		BUFFER_COPY_STRING_CONST(hctx->write_buffer, "GET ");
-		break;
-	case HTTP_METHOD_POST:
-		BUFFER_COPY_STRING_CONST(hctx->write_buffer, "POST ");
-		break;
-	case HTTP_METHOD_HEAD:
-		BUFFER_COPY_STRING_CONST(hctx->write_buffer, "HEAD ");
-		break;
-	default:
-		return -1;
-	}
+	buffer_copy_string(hctx->write_buffer, get_http_method_name(con->request.http_method));
+	BUFFER_APPEND_STRING_CONST(hctx->write_buffer, " ");
 	
 	buffer_append_string_buffer(hctx->write_buffer, con->request.uri);
 	BUFFER_APPEND_STRING_CONST(hctx->write_buffer, " HTTP/1.0\r\n");
@@ -460,8 +439,7 @@ static int proxy_create_env(server *srv, handler_ctx *hctx) {
 	
 	/* body */
 	
-	if (con->request.http_method == HTTP_METHOD_POST &&
-	    con->request.content_length) {
+	if (con->request.content_length) {
 		/* the buffer-string functions add an extra \0 at the end the memory-function don't */
 		hctx->write_buffer->used--;
 		buffer_append_memory(hctx->write_buffer, con->request.content->ptr, con->request.content_length);
@@ -799,16 +777,18 @@ static handler_t proxy_write_request(server *srv, handler_ctx *hctx) {
 
 #define PATCH(x) \
 	p->conf.x = s->x;
-static int mod_proxy_patch_connection(server *srv, connection *con, plugin_data *p, const char *stage, size_t stage_len) {
+static int mod_proxy_patch_connection(server *srv, connection *con, plugin_data *p) {
 	size_t i, j;
+	plugin_config *s = p->config_storage[0];
+	
+	PATCH(extensions);
+	PATCH(debug);
+	PATCH(balance);
 	
 	/* skip the first, the global context */
 	for (i = 1; i < srv->config_context->used; i++) {
 		data_config *dc = (data_config *)srv->config_context->data[i];
-		plugin_config *s = p->config_storage[i];
-		
-		/* not our stage */
-		if (!buffer_is_equal_string(dc->comp_key, stage, stage_len)) continue;
+		s = p->config_storage[i];
 		
 		/* condition didn't match */
 		if (!config_check_cond(srv, con, dc)) continue;
@@ -829,18 +809,6 @@ static int mod_proxy_patch_connection(server *srv, connection *con, plugin_data
 	
 	return 0;
 }
-
-static int mod_proxy_setup_connection(server *srv, connection *con, plugin_data *p) {
-	plugin_config *s = p->config_storage[0];
-	UNUSED(srv);
-	UNUSED(con);
-	
-	PATCH(extensions);
-	PATCH(debug);
-	PATCH(balance);
-	
-	return 0;
-}
 #undef PATCH
 
 SUBREQUEST_FUNC(mod_proxy_handle_subrequest) {
@@ -848,17 +816,10 @@ SUBREQUEST_FUNC(mod_proxy_handle_subrequest) {
 	
 	handler_ctx *hctx = con->plugin_ctx[p->id];
 	data_proxy *host;
-	size_t i;
 	
 	if (NULL == hctx) return HANDLER_GO_ON;
 
-	/* select the right config */
-	mod_proxy_setup_connection(srv, con, p);
-	for (i = 0; i < srv->config_patches->used; i++) {
-		buffer *patch = srv->config_patches->ptr[i];
-		
-		mod_proxy_patch_connection(srv, con, p, CONST_BUF_LEN(patch));
-	}
+	mod_proxy_patch_connection(srv, con, p);
 	
 	host = hctx->host;
 	
@@ -877,7 +838,7 @@ SUBREQUEST_FUNC(mod_proxy_handle_subrequest) {
 		host->is_disabled = 1;
 		host->disable_ts = srv->cur_ts;
 		
-		proxy_connection_cleanup(srv, hctx);
+		proxy_connection_close(srv, hctx);
 	
 		/* reset the enviroment and restart the sub-request */	
 		buffer_reset(con->physical.path);
@@ -906,23 +867,6 @@ SUBREQUEST_FUNC(mod_proxy_handle_subrequest) {
 	}
 }
 
-static handler_t proxy_connection_close(server *srv, handler_ctx *hctx) {
-	plugin_data *p;
-	connection  *con;
-	
-	if (NULL == hctx) return HANDLER_GO_ON;
-	
-	p    = hctx->plugin_data;
-	con  = hctx->remote_conn;
-	
-	if (con->mode != p->id) return HANDLER_GO_ON;
-	
-	proxy_connection_cleanup(srv, hctx);
-	
-	return HANDLER_FINISHED;
-}
-
-
 static handler_t proxy_handle_fdevent(void *s, void *ctx, int revents) {
 	server      *srv  = (server *)s;
 	handler_ctx *hctx = ctx;
@@ -945,7 +889,7 @@ static handler_t proxy_handle_fdevent(void *s, void *ctx, int revents) {
 			hctx->host->usage--;
 			
 			/* we are done */
-			proxy_connection_cleanup(srv, hctx);
+			proxy_connection_close(srv, hctx);
 			
 			joblist_append(srv, con);
 			return HANDLER_FINISHED;
@@ -1016,12 +960,10 @@ static handler_t proxy_handle_fdevent(void *s, void *ctx, int revents) {
 	} else if (revents & FDEVENT_ERR) {
 		/* kill all connections to the proxy process */
 		
-		log_error_write(srv, __FILE__, __LINE__, "s", "proxy-FDEVENT_ERR");
+		log_error_write(srv, __FILE__, __LINE__, "sd", "proxy-FDEVENT_ERR, but no HUP", revents);
 
 		joblist_append(srv, con);
 		proxy_connection_close(srv, hctx);
-
-		return HANDLER_ERROR;
 	}
 	
 	return HANDLER_FINISHED;
@@ -1030,9 +972,9 @@ static handler_t proxy_handle_fdevent(void *s, void *ctx, int revents) {
 static handler_t mod_proxy_check_extension(server *srv, connection *con, void *p_d) {
 	plugin_data *p = p_d;
 	size_t s_len;
-	unsigned long last_max = ~0L;
+	unsigned long last_max = ULONG_MAX;
 	int ndx = -1;
-	size_t k, i;
+	size_t k;
 	buffer *fn;
 	data_array *extension = NULL;
 	size_t path_info_offset;
@@ -1040,13 +982,7 @@ static handler_t mod_proxy_check_extension(server *srv, connection *con, void *p
 	/* Possibly, we processed already this request */
 	if (con->file_started == 1) return HANDLER_GO_ON;
 	
-	/* select the right config */
-	mod_proxy_setup_connection(srv, con, p);
-	for (i = 0; i < srv->config_patches->used; i++) {
-		buffer *patch = srv->config_patches->ptr[i];
-		
-		mod_proxy_patch_connection(srv, con, p, CONST_BUF_LEN(patch));
-	}
+	mod_proxy_patch_connection(srv, con, p);
 	
 	fn = con->uri.path;
 
@@ -1238,7 +1174,9 @@ static handler_t mod_proxy_check_extension(server *srv, connection *con, void *p
 static handler_t mod_proxy_connection_close_callback(server *srv, connection *con, void *p_d) {
 	plugin_data *p = p_d;
 	
-	return proxy_connection_close(srv, con->plugin_ctx[p->id]);
+	proxy_connection_close(srv, con->plugin_ctx[p->id]);
+
+	return HANDLER_GO_ON;
 }
 
 /**
@@ -1284,17 +1222,17 @@ TRIGGER_FUNC(mod_proxy_trigger) {
 
 
 int mod_proxy_plugin_init(plugin *p) {
-	p->version     = LIGHTTPD_VERSION_ID;
+	p->version      = LIGHTTPD_VERSION_ID;
 	p->name         = buffer_init_string("proxy");
 
 	p->init         = mod_proxy_init;
 	p->cleanup      = mod_proxy_free;
 	p->set_defaults = mod_proxy_set_defaults;
-	p->connection_reset        = mod_proxy_connection_reset;
-	p->handle_connection_close = mod_proxy_connection_close_callback;
+	p->connection_reset        = mod_proxy_connection_close_callback; /* end of req-resp cycle */
+	p->handle_connection_close = mod_proxy_connection_close_callback; /* end of client connection */
 	p->handle_uri_clean        = mod_proxy_check_extension;
 	p->handle_subrequest       = mod_proxy_handle_subrequest;
-	p->handle_trigger       = mod_proxy_trigger;
+	p->handle_trigger          = mod_proxy_trigger;
 	
 	p->data         = NULL;
 	
