@@ -31,6 +31,7 @@ typedef struct {
 
 typedef struct {
 	rewrite_rule_buffer *rewrite;
+	data_config *context; /* to which apply me */
 } plugin_config;
 
 typedef struct {
@@ -237,13 +238,13 @@ SETDEFAULTS_FUNC(mod_rewrite_set_defaults) {
 	if (!p) return HANDLER_ERROR;
 	
 	/* 0 */
-	p->config_storage = malloc(srv->config_context->used * sizeof(specific_config *));
+	p->config_storage = calloc(1, srv->config_context->used * sizeof(specific_config *));
 	
 	for (i = 0; i < srv->config_context->used; i++) {
 		plugin_config *s;
 		array *ca;
 		
-		s = malloc(sizeof(plugin_config));
+		s = calloc(1, sizeof(plugin_config));
 		s->rewrite   = rewrite_rule_buffer_init();
 		
 		cv[0].destination = s->rewrite;
@@ -266,16 +267,17 @@ SETDEFAULTS_FUNC(mod_rewrite_set_defaults) {
 	return HANDLER_GO_ON;
 }
 #ifdef HAVE_PCRE_H
-static int mod_rewrite_patch_connection(server *srv, connection *con, plugin_data *p, const char *stage, size_t stage_len) {
+static int mod_rewrite_patch_connection(server *srv, connection *con, plugin_data *p) {
 	size_t i, j;
+	plugin_config *s = p->config_storage[0];
+	p->conf.rewrite = s->rewrite;
 	
 	/* skip the first, the global context */
 	for (i = 1; i < srv->config_context->used; i++) {
 		data_config *dc = (data_config *)srv->config_context->data[i];
-		plugin_config *s = p->config_storage[i];
+		s = p->config_storage[i];
 		
-		/* not our stage */
-		if (!buffer_is_equal_string(dc->comp_key, stage, stage_len)) continue;
+		if (COMP_HTTP_URL == dc->comp) continue;
 		
 		/* condition didn't match */
 		if (!config_check_cond(srv, con, dc)) continue;
@@ -298,17 +300,6 @@ static int mod_rewrite_patch_connection(server *srv, connection *con, plugin_dat
 	
 	return 0;
 }
-
-static int mod_rewrite_setup_connection(server *srv, connection *con, plugin_data *p) {
-	plugin_config *s = p->config_storage[0];
-	
-	UNUSED(srv);
-	UNUSED(con);
-		
-	p->conf.rewrite = s->rewrite;
-	
-	return 0;
-}
 #endif
 URIHANDLER_FUNC(mod_rewrite_con_reset) {
 	plugin_data *p = p_d;
@@ -340,7 +331,8 @@ URIHANDLER_FUNC(mod_rewrite_uri_handler) {
 		hctx = con->plugin_ctx[p->id];
 		
 		if (hctx->loops++ > 100) {
-			log_error_write(srv, __FILE__, __LINE__,  "s",  "ENDLESS LOOP IN rewrite-rule DETECTED ... aborting request, perhaps you want to use url.rewrite instead of url.rewrite-repeat");
+			log_error_write(srv, __FILE__, __LINE__,  "s",  
+					"ENDLESS LOOP IN rewrite-rule DETECTED ... aborting request, perhaps you want to use url.rewrite-once instead of url.rewrite-repeat");
 			
 			return HANDLER_ERROR;
 		}
@@ -348,12 +340,9 @@ URIHANDLER_FUNC(mod_rewrite_uri_handler) {
 		if (hctx->state == REWRITE_STATE_FINISHED) return HANDLER_GO_ON;
 	}
 	
-	mod_rewrite_setup_connection(srv, con, p);
-	for (i = 0; i < srv->config_patches->used; i++) {
-		buffer *patch = srv->config_patches->ptr[i];
-		
-		mod_rewrite_patch_connection(srv, con, p, CONST_BUF_LEN(patch));
-	}
+	mod_rewrite_patch_connection(srv, con, p);
+
+	if (!p->conf.rewrite) return HANDLER_GO_ON;
 	
 	buffer_copy_string_buffer(p->match_buf, con->request.uri);
 	
@@ -390,7 +379,7 @@ URIHANDLER_FUNC(mod_rewrite_uri_handler) {
 			
 			start = 0; end = pattern_len;
 			for (k = 0; k < pattern_len; k++) {
-				if (pattern[k] == '$' &&
+				if ((pattern[k] == '$' || pattern[k] == '%') &&
 				    isdigit((unsigned char)pattern[k + 1])) {
 					/* got one */
 					
@@ -400,9 +389,13 @@ URIHANDLER_FUNC(mod_rewrite_uri_handler) {
 					
 					buffer_append_string_len(con->request.uri, pattern + start, end - start);
 					
-					/* n is always larger than 0 */
-					if (num < (size_t)n) {
-						buffer_append_string(con->request.uri, list[num]);
+					if (pattern[k] == '$') {
+						/* n is always > 0 */
+						if (num < (size_t)n) {
+							buffer_append_string(con->request.uri, list[num]);
+						}
+					} else {
+						config_append_cond_match_buffer(con, p->conf.context, con->request.uri, num);
 					}
 					
 					k++;
