@@ -311,8 +311,6 @@ typedef struct {
 	int       fd;        /* fd to the scgi process */
 	int       fde_ndx;   /* index into the fd-event buffer */
 
-	size_t    path_info_offset; /* start of path_info in uri.path */
-	
 	pid_t     pid;
 	int       got_proc;
 	
@@ -670,8 +668,8 @@ static int scgi_spawn_connection(server *srv,
 			
 			if (NULL == (he = gethostbyname(host->host->ptr))) {
 				log_error_write(srv, __FILE__, __LINE__, 
-						"ssb", "gethostbyname failed: ", 
-						hstrerror(h_errno), host->host);
+						"sdb", "gethostbyname failed: ", 
+						h_errno, host->host);
 				return -1;
 			}
 			
@@ -2592,16 +2590,17 @@ static handler_t scgi_handle_fdevent(void *s, void *ctx, int revents) {
 }
 #define PATCH(x) \
 	p->conf.x = s->x;
-static int scgi_patch_connection(server *srv, connection *con, plugin_data *p, const char *stage, size_t stage_len) {
+static int scgi_patch_connection(server *srv, connection *con, plugin_data *p) {
 	size_t i, j;
+	plugin_config *s = p->config_storage[0];
+	
+	PATCH(exts);
+	PATCH(debug);
 	
 	/* skip the first, the global context */
 	for (i = 1; i < srv->config_context->used; i++) {
 		data_config *dc = (data_config *)srv->config_context->data[i];
-		plugin_config *s = p->config_storage[i];
-		
-		/* not our stage */
-		if (!buffer_is_equal_string(dc->comp_key, stage, stage_len)) continue;
+		s = p->config_storage[i];
 		
 		/* condition didn't match */
 		if (!config_check_cond(srv, con, dc)) continue;
@@ -2620,17 +2619,6 @@ static int scgi_patch_connection(server *srv, connection *con, plugin_data *p, c
 	
 	return 0;
 }
-
-static int scgi_setup_connection(server *srv, connection *con, plugin_data *p) {
-	plugin_config *s = p->config_storage[0];
-	UNUSED(srv);
-	UNUSED(con);
-	
-	PATCH(exts);
-	PATCH(debug);
-	
-	return 0;
-}
 #undef PATCH
 
 
@@ -2639,10 +2627,9 @@ static handler_t scgi_check_extension(server *srv, connection *con, void *p_d, i
 	size_t s_len;
 	int used = -1;
 	int ndx;
-	size_t k, i;
+	size_t k;
 	buffer *fn;
 	scgi_extension *extension = NULL;
-	size_t path_info_offset;
 	
 	/* Possibly, we processed already this request */
 	if (con->file_started == 1) return HANDLER_GO_ON;
@@ -2655,16 +2642,8 @@ static handler_t scgi_check_extension(server *srv, connection *con, void *p_d, i
 
 	s_len = fn->used - 1;
 	
-	/* select the right config */
-	scgi_setup_connection(srv, con, p);
-	for (i = 0; i < srv->config_patches->used; i++) {
-		buffer *patch = srv->config_patches->ptr[i];
-		
-		scgi_patch_connection(srv, con, p, CONST_BUF_LEN(patch));
-	}
+	scgi_patch_connection(srv, con, p);
 
-	path_info_offset = 0;
-	
 	/* check if extension matches */
 	for (k = 0; k < p->conf.exts->used; k++) {
 		size_t ct_len;
@@ -2679,13 +2658,6 @@ static handler_t scgi_check_extension(server *srv, connection *con, void *p_d, i
 		
 		/* check extension in the form "/scgi_pattern" */
 		if (*(extension->key->ptr) == '/' && strncmp(fn->ptr, extension->key->ptr, ct_len) == 0) {
-			if (s_len > ct_len + 1) {
-				char *pi_offset;
-				
-				if (0 != (pi_offset = strchr(fn->ptr + ct_len + 1, '/'))) {
-					path_info_offset = pi_offset - fn->ptr;
-				}
-			}
 			break;
 		} else if (0 == strncmp(fn->ptr + s_len - ct_len, extension->key->ptr, ct_len)) {
 			/* check extension in the form ".fcg" */
@@ -2725,9 +2697,10 @@ static handler_t scgi_check_extension(server *srv, connection *con, void *p_d, i
 		if (uri_path_handler) {
 			if (host->check_local == 0) {
 				handler_ctx *hctx;
+				char *pathinfo;
+
 				hctx = handler_ctx_init();
 				
-				hctx->path_info_offset = path_info_offset;
 				hctx->remote_conn      = con;
 				hctx->plugin_data      = p;
 				hctx->host             = host;
@@ -2741,13 +2714,48 @@ static handler_t scgi_check_extension(server *srv, connection *con, void *p_d, i
 				host->load++;
 				
 				con->mode = p->id;
+
+				if (con->conf.log_request_handling) {
+					log_error_write(srv, __FILE__, __LINE__, "s", "handling it in mod_scgi");
+				}
+
+				/* the prefix is the SCRIPT_NAME, 
+				 * everthing from start to the next slash
+				 * this is important for check-local = "disable"
+				 * 
+				 * if prefix = /admin.fcgi
+				 * 
+				 * /admin.fcgi/foo/bar
+				 * 
+				 * SCRIPT_NAME = /admin.fcgi
+				 * PATH_INFO   = /foo/bar
+				 * 
+				 * if prefix = /fcgi-bin/
+				 * 
+				 * /fcgi-bin/foo/bar
+				 * 
+				 * SCRIPT_NAME = /fcgi-bin/foo
+				 * PATH_INFO   = /bar
+				 * 
+				 */
+				
+				/* the rewrite is only done for /prefix/? matches */
+				if (extension->key->ptr[0] == '/' &&
+				    con->uri.path->used > extension->key->used &&
+				    NULL != (pathinfo = strchr(con->uri.path->ptr + extension->key->used - 1, '/'))) {
+					/* rewrite uri.path and pathinfo */ 
+					
+					buffer_copy_string(con->request.pathinfo, pathinfo);
+					
+					con->uri.path->used -= con->request.pathinfo->used - 1;
+					con->uri.path->ptr[con->uri.path->used - 1] = '\0';
+				}
 			}
 			return HANDLER_GO_ON;
 		} else {
 			handler_ctx *hctx;
 			hctx = handler_ctx_init();
 			
-			hctx->path_info_offset = path_info_offset;
 			hctx->remote_conn      = con;
 			hctx->plugin_data      = p;
 			hctx->host             = host;
@@ -2762,7 +2770,11 @@ static handler_t scgi_check_extension(server *srv, connection *con, void *p_d, i
 			
 			con->mode = p->id;
 			
-			return HANDLER_FINISHED;
+			if (con->conf.log_request_handling) {
+				log_error_write(srv, __FILE__, __LINE__, "s", "handling it in mod_fastcgi");
+			}
+
+			return HANDLER_GO_ON;
 		}
 	} else {
 		/* no handler found */
