@@ -106,22 +106,16 @@ SETDEFAULTS_FUNC(mod_secdownload_set_defaults) {
 	
 	if (!p) return HANDLER_ERROR;
 	
-	p->config_storage = malloc(srv->config_context->used * sizeof(specific_config *));
+	p->config_storage = calloc(1, srv->config_context->used * sizeof(specific_config *));
 	
 	for (i = 0; i < srv->config_context->used; i++) {
 		plugin_config *s;
 		
-		s = malloc(sizeof(plugin_config));
+		s = calloc(1, sizeof(plugin_config));
 		s->secret        = buffer_init();
 		s->doc_root      = buffer_init();
 		s->uri_prefix    = buffer_init();
-		s->timeout = 0;
-		
-		/* set global default */
-		if (i == 0) {
-			s->timeout = 60;
-			buffer_copy_string(s->uri_prefix, "/");
-		}
+		s->timeout       = 60;
 		
 		cv[0].destination = s->secret;
 		cv[1].destination = s->doc_root;
@@ -165,16 +159,19 @@ int is_hex_len(const char *str, size_t len) {
 
 #define PATCH(x) \
 	p->conf.x = s->x;
-static int mod_secdownload_patch_connection(server *srv, connection *con, plugin_data *p, const char *stage, size_t stage_len) {
+static int mod_secdownload_patch_connection(server *srv, connection *con, plugin_data *p) {
 	size_t i, j;
+	plugin_config *s = p->config_storage[0];
+	
+	PATCH(secret);
+	PATCH(doc_root);
+	PATCH(uri_prefix);
+	PATCH(timeout);
 	
 	/* skip the first, the global context */
 	for (i = 1; i < srv->config_context->used; i++) {
 		data_config *dc = (data_config *)srv->config_context->data[i];
-		plugin_config *s = p->config_storage[i];
-		
-		/* not our stage */
-		if (!buffer_is_equal_string(dc->comp_key, stage, stage_len)) continue;
+		s = p->config_storage[i];
 		
 		/* condition didn't match */
 		if (!config_check_cond(srv, con, dc)) continue;
@@ -197,19 +194,6 @@ static int mod_secdownload_patch_connection(server *srv, connection *con, plugin
 	
 	return 0;
 }
-
-static int mod_secdownload_setup_connection(server *srv, connection *con, plugin_data *p) {
-	plugin_config *s = p->config_storage[0];
-	UNUSED(srv);
-	UNUSED(con);
-
-	PATCH(secret);
-	PATCH(doc_root);
-	PATCH(uri_prefix);
-	PATCH(timeout);
-	
-	return 0;
-}
 #undef PATCH
 
 
@@ -223,12 +207,9 @@ URIHANDLER_FUNC(mod_secdownload_uri_handler) {
 	
 	if (con->uri.path->used == 0) return HANDLER_GO_ON;
 	
-	mod_secdownload_setup_connection(srv, con, p);
-	for (i = 0; i < srv->config_patches->used; i++) {
-		buffer *patch = srv->config_patches->ptr[i];
-		
-		mod_secdownload_patch_connection(srv, con, p, CONST_BUF_LEN(patch));
-	}
+	mod_secdownload_patch_connection(srv, con, p);
+
+	if (buffer_is_empty(p->conf.uri_prefix)) return HANDLER_GO_ON;
 	
 	if (buffer_is_empty(p->conf.secret)) {
 		log_error_write(srv, __FILE__, __LINE__, "s",
@@ -300,10 +281,12 @@ URIHANDLER_FUNC(mod_secdownload_uri_handler) {
 	/* starting with the last / we should have relative-path to the docroot
 	 */
 	
-	buffer_copy_string_buffer(con->physical.path, p->conf.doc_root);
-	buffer_append_string(con->physical.path, rel_uri);
+	buffer_copy_string_buffer(con->physical.doc_root, p->conf.doc_root);
+	buffer_copy_string(con->physical.rel_path, rel_uri);
+	buffer_copy_string_buffer(con->physical.path, con->physical.doc_root);
+	buffer_append_string_buffer(con->physical.path, con->physical.rel_path);
 	
-	return HANDLER_COMEBACK;
+	return HANDLER_GO_ON;
 }
 
 /* this function is called at dlopen() time and inits the callbacks */
@@ -313,7 +296,7 @@ int mod_secdownload_plugin_init(plugin *p) {
 	p->name        = buffer_init_string("secdownload");
 	
 	p->init        = mod_secdownload_init;
-	p->handle_uri_clean  = mod_secdownload_uri_handler;
+	p->handle_physical  = mod_secdownload_uri_handler;
 	p->set_defaults  = mod_secdownload_set_defaults;
 	p->cleanup     = mod_secdownload_free;
 	
