@@ -79,12 +79,12 @@ SETDEFAULTS_FUNC(mod_setenv_set_defaults) {
 	
 	if (!p) return HANDLER_ERROR;
 	
-	p->config_storage = malloc(srv->config_context->used * sizeof(specific_config *));
+	p->config_storage = calloc(1, srv->config_context->used * sizeof(specific_config *));
 	
 	for (i = 0; i < srv->config_context->used; i++) {
 		plugin_config *s;
 		
-		s = malloc(sizeof(plugin_config));
+		s = calloc(1, sizeof(plugin_config));
 		s->request_header   = array_init();
 		s->response_header  = array_init();
 		s->environment      = array_init();
@@ -105,16 +105,18 @@ SETDEFAULTS_FUNC(mod_setenv_set_defaults) {
 
 #define PATCH(x) \
 	p->conf.x = s->x;
-static int mod_setenv_patch_connection(server *srv, connection *con, plugin_data *p, const char *stage, size_t stage_len) {
+static int mod_setenv_patch_connection(server *srv, connection *con, plugin_data *p) {
 	size_t i, j;
+	plugin_config *s = p->config_storage[0];
+	
+	PATCH(request_header);
+	PATCH(response_header);
+	PATCH(environment);
 	
 	/* skip the first, the global context */
 	for (i = 1; i < srv->config_context->used; i++) {
 		data_config *dc = (data_config *)srv->config_context->data[i];
-		plugin_config *s = p->config_storage[i];
-		
-		/* not our stage */
-		if (!buffer_is_equal_string(dc->comp_key, stage, stage_len)) continue;
+		s = p->config_storage[i];
 		
 		/* condition didn't match */
 		if (!config_check_cond(srv, con, dc)) continue;
@@ -135,30 +137,13 @@ static int mod_setenv_patch_connection(server *srv, connection *con, plugin_data
 	
 	return 0;
 }
-
-static int mod_setenv_setup_connection(server *srv, connection *con, plugin_data *p) {
-	plugin_config *s = p->config_storage[0];
-	UNUSED(srv);
-	UNUSED(con);
-		
-	PATCH(request_header);
-	PATCH(response_header);
-	PATCH(environment);
-	
-	return 0;
-}
 #undef PATCH
 
 URIHANDLER_FUNC(mod_setenv_uri_handler) {
 	plugin_data *p = p_d;
 	size_t k, i;
 	
-	mod_setenv_setup_connection(srv, con, p);
-	for (i = 0; i < srv->config_patches->used; i++) {
-		buffer *patch = srv->config_patches->ptr[i];
-		
-		mod_setenv_patch_connection(srv, con, p, CONST_BUF_LEN(patch));
-	}
+	mod_setenv_patch_connection(srv, con, p);
 
 	for (k = 0; k < p->conf.request_header->used; k++) {
 		data_string *ds = (data_string *)p->conf.request_header->data[k];
