@@ -2,6 +2,7 @@
 
 #include <ctype.h>
 #include <stdlib.h>
+#include <stdio.h>
 #include <string.h>
 #include <errno.h>
 #include <time.h>
@@ -10,6 +11,7 @@
 #include "base.h"
 #include "log.h"
 #include "buffer.h"
+#include "stat_cache.h"
 
 #include "plugin.h"
 #include "stream.h"
@@ -93,12 +95,12 @@ SETDEFAULTS_FUNC(mod_ssi_set_defaults) {
 	
 	if (!p) return HANDLER_ERROR;
 	
-	p->config_storage = malloc(srv->config_context->used * sizeof(specific_config *));
+	p->config_storage = calloc(1, srv->config_context->used * sizeof(specific_config *));
 	
 	for (i = 0; i < srv->config_context->used; i++) {
 		plugin_config *s;
 		
-		s = malloc(sizeof(plugin_config));
+		s = calloc(1, sizeof(plugin_config));
 		s->ssi_extension  = array_init();
 		
 		cv[0].destination = s->ssi_extension;
@@ -171,9 +173,15 @@ static int ssi_env_add_request_headers(server *srv, connection *con, plugin_data
 			
 			buffer_prepare_append(srv->tmp_buf, ds->key->used + 2);
 			for (j = 0; j < ds->key->used - 1; j++) {
-				srv->tmp_buf->ptr[srv->tmp_buf->used++] = 
-					isalpha((unsigned char)ds->key->ptr[j]) ? 
-					toupper((unsigned char)ds->key->ptr[j]) : '_';
+				char c = '_';
+				if (light_isalpha(ds->key->ptr[j])) {
+					/* upper-case */
+					c = ds->key->ptr[j] & ~32;
+				} else if (light_isdigit(ds->key->ptr[j])) {
+					/* copy */
+					c = ds->key->ptr[j];
+				}
+				srv->tmp_buf->ptr[srv->tmp_buf->used++] = c;
 			}
 			srv->tmp_buf->ptr[srv->tmp_buf->used] = '\0';
 			
@@ -313,6 +321,7 @@ static int process_ssi_stmt(server *srv, connection *con, plugin_data *p,
 		/* echo */
 		int var = 0, enc = 0;
 		const char *var_val = NULL;
+		stat_cache_entry *sce = NULL;
 		
 		struct { 
 			const char *var;
@@ -376,6 +385,8 @@ static int process_ssi_stmt(server *srv, connection *con, plugin_data *p,
 					l[1], "var is missing");
 			break;
 		}
+
+		stat_cache_get_entry(srv, con, con->physical.path, &sce);
 		
 		switch(var) {
 		case SSI_ECHO_USER_NAME: {
@@ -383,18 +394,18 @@ static int process_ssi_stmt(server *srv, connection *con, plugin_data *p,
 			
 			b = chunkqueue_get_append_buffer(con->write_queue);
 #ifdef HAVE_PWD_H
-			if (NULL == (pw = getpwuid(con->fce->st.st_uid))) {
-				buffer_copy_long(b, con->fce->st.st_uid);
+			if (NULL == (pw = getpwuid(sce->st.st_uid))) {
+				buffer_copy_long(b, sce->st.st_uid);
 			} else {
 				buffer_copy_string(b, pw->pw_name);
 			}
 #else
-			buffer_copy_long(b, con->fce->st.st_uid);
+			buffer_copy_long(b, sce->st.st_uid);
 #endif
 			break;
 		}
 		case SSI_ECHO_LAST_MODIFIED:	{
-			time_t t = con->fce->st.st_mtime;
+			time_t t = sce->st.st_mtime;
 			
 			b = chunkqueue_get_append_buffer(con->write_queue);
 			if (0 == strftime(buf, sizeof(buf), p->timefmt->ptr, localtime(&t))) {
@@ -980,16 +991,16 @@ static int mod_ssi_handle_request(server *srv, connection *con, plugin_data *p)
 
 #define PATCH(x) \
 	p->conf.x = s->x;
-static int mod_ssi_patch_connection(server *srv, connection *con, plugin_data *p, const char *stage, size_t stage_len) {
+static int mod_ssi_patch_connection(server *srv, connection *con, plugin_data *p) {
 	size_t i, j;
+	plugin_config *s = p->config_storage[0];
+	
+	PATCH(ssi_extension);
 	
 	/* skip the first, the global context */
 	for (i = 1; i < srv->config_context->used; i++) {
 		data_config *dc = (data_config *)srv->config_context->data[i];
-		plugin_config *s = p->config_storage[i];
-		
-		/* not our stage */
-		if (!buffer_is_equal_string(dc->comp_key, stage, stage_len)) continue;
+		s = p->config_storage[i];
 		
 		/* condition didn't match */
 		if (!config_check_cond(srv, con, dc)) continue;
@@ -1006,30 +1017,15 @@ static int mod_ssi_patch_connection(server *srv, connection *con, plugin_data *p
 	
 	return 0;
 }
-
-static int mod_ssi_setup_connection(server *srv, connection *con, plugin_data *p) {
-	plugin_config *s = p->config_storage[0];
-	UNUSED(srv);
-	UNUSED(con);
-		
-	PATCH(ssi_extension);
-	
-	return 0;
-}
 #undef PATCH
 
 URIHANDLER_FUNC(mod_ssi_physical_path) {
 	plugin_data *p = p_d;
-	size_t k, i;
+	size_t k;
 	
 	if (con->physical.path->used == 0) return HANDLER_GO_ON;
 	
-	mod_ssi_setup_connection(srv, con, p);
-	for (i = 0; i < srv->config_patches->used; i++) {
-		buffer *patch = srv->config_patches->ptr[i];
-		
-		mod_ssi_patch_connection(srv, con, p, CONST_BUF_LEN(patch));
-	}
+	mod_ssi_patch_connection(srv, con, p);
 	
 	for (k = 0; k < p->conf.ssi_extension->used; k++) {
 		data_string *ds = (data_string *)p->conf.ssi_extension->data[k];
@@ -1059,7 +1055,7 @@ int mod_ssi_plugin_init(plugin *p) {
 	p->name        = buffer_init_string("ssi");
 	
 	p->init        = mod_ssi_init;
-	p->handle_physical_path = mod_ssi_physical_path;
+	p->handle_subrequest_start = mod_ssi_physical_path;
 	p->set_defaults  = mod_ssi_set_defaults;
 	p->cleanup     = mod_ssi_free;
 	
