@@ -55,6 +55,7 @@ typedef struct {
 #endif
 	
 	unsigned short trigger_timeout;
+	unsigned short debug;
 } plugin_config;
 
 typedef struct {
@@ -90,6 +91,8 @@ FREE_FUNC(mod_trigger_b4_dl_free) {
 		size_t i;
 		for (i = 0; i < srv->config_context->used; i++) {
 			plugin_config *s = p->config_storage[i];
+
+			if (!s) continue;
 			
 			buffer_free(s->db_filename);
 			buffer_free(s->download_url);
@@ -137,6 +140,7 @@ SETDEFAULTS_FUNC(mod_trigger_b4_dl_set_defaults) {
 		{ "trigger-before-download.trigger-timeout", NULL, T_CONFIG_SHORT, T_CONFIG_SCOPE_CONNECTION },        /* 4 */
 		{ "trigger-before-download.memcache-hosts",  NULL, T_CONFIG_ARRAY, T_CONFIG_SCOPE_CONNECTION },        /* 5 */
 		{ "trigger-before-download.memcache-namespace", NULL, T_CONFIG_STRING, T_CONFIG_SCOPE_CONNECTION },    /* 6 */
+		{ "trigger-before-download.debug",           NULL, T_CONFIG_BOOLEAN, T_CONFIG_SCOPE_CONNECTION },      /* 7 */
 		{ NULL,                        NULL, T_CONFIG_UNSET, T_CONFIG_SCOPE_UNSET }
 	};
 	
@@ -146,7 +150,6 @@ SETDEFAULTS_FUNC(mod_trigger_b4_dl_set_defaults) {
 	
 	for (i = 0; i < srv->config_context->used; i++) {
 		plugin_config *s;
-		size_t k;
 #if defined(HAVE_PCRE_H)
 		const char *errptr;
 		int erroff;
@@ -167,6 +170,7 @@ SETDEFAULTS_FUNC(mod_trigger_b4_dl_set_defaults) {
 		cv[4].destination = &(s->trigger_timeout);
 		cv[5].destination = s->mc_hosts;
 		cv[6].destination = s->mc_namespace;
+		cv[7].destination = &(s->debug);
 		
 		p->config_storage[i] = s;
 	
@@ -209,6 +213,7 @@ SETDEFAULTS_FUNC(mod_trigger_b4_dl_set_defaults) {
 
 		if (s->mc_hosts->used) {
 #if defined(HAVE_MEMCACHE_H)
+			size_t k;
 			s->mc = mc_new();
 		
 			for (k = 0; k < s->mc_hosts->used; k++) {
@@ -242,16 +247,29 @@ SETDEFAULTS_FUNC(mod_trigger_b4_dl_set_defaults) {
 
 #define PATCH(x) \
 	p->conf.x = s->x;
-static int mod_trigger_b4_dl_patch_connection(server *srv, connection *con, plugin_data *p, const char *stage, size_t stage_len) {
+static int mod_trigger_b4_dl_patch_connection(server *srv, connection *con, plugin_data *p) {
 	size_t i, j;
+	plugin_config *s = p->config_storage[0];
+	
+#if defined(HAVE_GDBM)
+	PATCH(db);
+#endif	
+#if defined(HAVE_PCRE_H)
+	PATCH(download_regex);
+	PATCH(trigger_regex);
+#endif	
+	PATCH(trigger_timeout);
+	PATCH(deny_url);
+	PATCH(mc_namespace);
+	PATCH(debug);
+#if defined(HAVE_MEMCACHE_H)
+	PATCH(mc);
+#endif
 	
 	/* skip the first, the global context */
 	for (i = 1; i < srv->config_context->used; i++) {
 		data_config *dc = (data_config *)srv->config_context->data[i];
-		plugin_config *s = p->config_storage[i];
-		
-		/* not our stage */
-		if (!buffer_is_equal_string(dc->comp_key, stage, stage_len)) continue;
+		s = p->config_storage[i];
 		
 		/* condition didn't match */
 		if (!config_check_cond(srv, con, dc)) continue;
@@ -274,6 +292,8 @@ static int mod_trigger_b4_dl_patch_connection(server *srv, connection *con, plug
 #endif
 			} else if (buffer_is_equal_string(du->key, CONST_STR_LEN("trigger-before-download.trigger-timeout"))) {
 				PATCH(trigger_timeout);
+			} else if (buffer_is_equal_string(du->key, CONST_STR_LEN("trigger-before-download.debug"))) {
+				PATCH(debug);
 			} else if (buffer_is_equal_string(du->key, CONST_STR_LEN("trigger-before-download.deny-url"))) {
 				PATCH(deny_url);
 			} else if (buffer_is_equal_string(du->key, CONST_STR_LEN("trigger-before-download.memcache-namespace"))) {
@@ -288,30 +308,10 @@ static int mod_trigger_b4_dl_patch_connection(server *srv, connection *con, plug
 	
 	return 0;
 }
-
-static int mod_trigger_b4_dl_setup_connection(server *srv, connection *con, plugin_data *p) {
-	plugin_config *s = p->config_storage[0];
-	UNUSED(srv);
-	UNUSED(con);
-#if defined(HAVE_GDBM_H)
-	PATCH(db);
-#endif
-#if defined(HAVE_PCRE_H)
-	PATCH(download_regex);
-	PATCH(trigger_regex);
-#endif
-	PATCH(trigger_timeout);
-	PATCH(deny_url);
-#if defined(HAVE_MEMCACHE_H)
-	PATCH(mc);
-#endif	
-	return 0;
-}
 #undef PATCH
 
 URIHANDLER_FUNC(mod_trigger_b4_dl_uri_handler) {
 	plugin_data *p = p_d;
-	size_t i;
 	const char *remote_ip;
 	data_string *ds;
 
@@ -322,12 +322,7 @@ URIHANDLER_FUNC(mod_trigger_b4_dl_uri_handler) {
 	
 	if (con->uri.path->used == 0) return HANDLER_GO_ON;
 	
-	mod_trigger_b4_dl_setup_connection(srv, con, p);
-	for (i = 0; i < srv->config_patches->used; i++) {
-		buffer *patch = srv->config_patches->ptr[i];
-		
-		mod_trigger_b4_dl_patch_connection(srv, con, p, CONST_BUF_LEN(patch));
-	}
+	mod_trigger_b4_dl_patch_connection(srv, con, p);
 	
 	if (!p->conf.trigger_regex || !p->conf.download_regex) return HANDLER_GO_ON;
 	
@@ -355,7 +350,11 @@ URIHANDLER_FUNC(mod_trigger_b4_dl_uri_handler) {
 	} else {
 		remote_ip = inet_ntop_cache_get_ip(srv, &(con->dst_addr));
 	}
-	
+
+	if (p->conf.debug) {
+		log_error_write(srv, __FILE__, __LINE__, "ss", "(debug) remote-ip:", remote_ip);
+	}
+		
 	/* check if URL is a trigger -> insert IP into DB */
 	if ((n = pcre_exec(p->conf.trigger_regex, NULL, con->uri.path->ptr, con->uri.path->used - 1, 0, 0, ovec, 3 * N)) < 0) {
 		if (n != PCRE_ERROR_NOMATCH) {
@@ -384,6 +383,7 @@ URIHANDLER_FUNC(mod_trigger_b4_dl_uri_handler) {
 # endif
 # if defined(HAVE_MEMCACHE_H)		
 		if (p->conf.mc) {
+			size_t i;
 			buffer_copy_string_buffer(p->tmp_buf, p->conf.mc_namespace);
 			buffer_append_string(p->tmp_buf, remote_ip);
 			
@@ -391,6 +391,10 @@ URIHANDLER_FUNC(mod_trigger_b4_dl_uri_handler) {
 				if (p->tmp_buf->ptr[i] == ' ') p->tmp_buf->ptr[i] = '-';
 			}
 			
+			if (p->conf.debug) {
+				log_error_write(srv, __FILE__, __LINE__, "sb", "(debug) triggered IP:", p->tmp_buf);
+			}
+
 			if (0 != mc_set(p->conf.mc, 
 					CONST_BUF_LEN(p->tmp_buf),
 					(char *)&(srv->cur_ts), sizeof(srv->cur_ts),
@@ -464,6 +468,7 @@ URIHANDLER_FUNC(mod_trigger_b4_dl_uri_handler) {
 # if defined(HAVE_MEMCACHE_H)		
 		if (p->conf.mc) {
 			void *r;
+			size_t i;
 			
 			buffer_copy_string_buffer(p->tmp_buf, p->conf.mc_namespace);
 			buffer_append_string(p->tmp_buf, remote_ip);
@@ -472,6 +477,10 @@ URIHANDLER_FUNC(mod_trigger_b4_dl_uri_handler) {
 				if (p->tmp_buf->ptr[i] == ' ') p->tmp_buf->ptr[i] = '-';
 			}
 			
+			if (p->conf.debug) {
+				log_error_write(srv, __FILE__, __LINE__, "sb", "(debug) checking IP:", p->tmp_buf);
+			}
+
 			/**
 			 * 
 			 * memcached is do expiration for us, as long as we can fetch it every thing is ok
