@@ -21,7 +21,7 @@
 #include "network.h"
 #include "fdevent.h"
 #include "log.h"
-#include "file_cache.h"
+#include "stat_cache.h"
 
 
 #ifndef UIO_MAXIOV
@@ -60,12 +60,12 @@ int network_write_chunkqueue_freebsdsendfile(server *srv, connection *con, chunk
 			for(num_chunks = 0, tc = c; tc && tc->type == MEM_CHUNK && num_chunks < UIO_MAXIOV; num_chunks++, tc = tc->next);
 			
 			for(tc = c, i = 0; i < num_chunks; tc = tc->next, i++) {
-				if (tc->data.mem->used == 0) {
-					chunks[i].iov_base = tc->data.mem->ptr;
+				if (tc->mem->used == 0) {
+					chunks[i].iov_base = tc->mem->ptr;
 					chunks[i].iov_len  = 0;
 				} else {
-					offset = tc->data.mem->ptr + tc->offset;
-					toSend = tc->data.mem->used - 1 - tc->offset;
+					offset = tc->mem->ptr + tc->offset;
+					toSend = tc->mem->used - 1 - tc->offset;
 					
 					chunks[i].iov_base = offset;
 					
@@ -136,42 +136,54 @@ int network_write_chunkqueue_freebsdsendfile(server *srv, connection *con, chunk
 		case FILE_CHUNK: {
 			off_t offset, r;
 			size_t toSend;
+			stat_cache_entry *sce = NULL;
+			int ifd;
 			
-			if (HANDLER_GO_ON != file_cache_get_entry(srv, con, c->data.file.name, &(con->fce))) {
+			if (HANDLER_ERROR == stat_cache_get_entry(srv, con, c->file.name, &sce)) {
 				log_error_write(srv, __FILE__, __LINE__, "sb",
-						strerror(errno), c->data.file.name);
+						strerror(errno), c->file.name);
 				return -1;
 			}
 			
-			offset = c->data.file.offset + c->offset;
-			toSend = c->data.file.length - c->offset;
-			
-			if (offset > con->fce->st.st_size) {
-				log_error_write(srv, __FILE__, __LINE__, "sb", "file was shrinked:", c->data.file.name);
+			offset = c->file.offset + c->offset;
+			/* limit the toSend to 2^31-1 bytes in a chunk */
+			toSend = c->file.length - c->offset > ((1 << 30) - 1) ? 
+				((1 << 30) - 1) : c->file.length - c->offset;
+				
+			if (offset > sce->st.st_size) {
+				log_error_write(srv, __FILE__, __LINE__, "sb", "file was shrinked:", c->file.name);
 				
 				return -1;
 			}
-
+			
+			if (-1 == (ifd = open(c->file.name->ptr, O_RDONLY))) {
+				log_error_write(srv, __FILE__, __LINE__, "ss", "open failed: ", strerror(errno));
+				
+				return -1;
+			}
+			
 			r = 0;
 			
 			/* FreeBSD sendfile() */
-			if (-1 == sendfile(con->fce->fd, fd, offset, toSend, NULL, &r, 0)) {
+			if (-1 == sendfile(ifd, fd, offset, toSend, NULL, &r, 0)) {
 				switch(errno) {
 				case EAGAIN:
 					break;
 				case ENOTCONN:
+					close(ifd);
 					return -2;
 				default:
 					log_error_write(srv, __FILE__, __LINE__, "ssd", "sendfile: ", strerror(errno), errno);
-					
+					close(ifd);
 					return -1;
 				}
 			}
-
+			close(ifd);
+			
 			c->offset += r;
 			con->bytes_written += r;
 			
-			if (c->offset == c->data.file.length) {
+			if (c->offset == c->file.length) {
 				chunk_finished = 1;
 			}
 			
