@@ -32,7 +32,6 @@ typedef enum {
 	PLUGIN_FUNC_UNSET,
 		PLUGIN_FUNC_HANDLE_URI_CLEAN, 
 		PLUGIN_FUNC_HANDLE_URI_RAW, 
-		PLUGIN_FUNC_HANDLE_PHYSICAL_PATH,
 		PLUGIN_FUNC_HANDLE_REQUEST_DONE,
 		PLUGIN_FUNC_HANDLE_CONNECTION_CLOSE,
 		PLUGIN_FUNC_HANDLE_TRIGGER,
@@ -41,6 +40,7 @@ typedef enum {
 		PLUGIN_FUNC_HANDLE_SUBREQUEST_START,
 		PLUGIN_FUNC_HANDLE_JOBLIST,
 		PLUGIN_FUNC_HANDLE_DOCROOT,
+		PLUGIN_FUNC_HANDLE_PHYSICAL,
 		PLUGIN_FUNC_CONNECTION_RESET,
 		PLUGIN_FUNC_INIT, 
 		PLUGIN_FUNC_CLEANUP,
@@ -61,7 +61,7 @@ static void plugin_free(plugin *p) {
 	int use_dlclose = 1;
 	if (p->name) buffer_free(p->name);
 #ifdef HAVE_VALGRIND_VALGRIND_H
-	if (RUNNING_ON_VALGRIND) use_dlclose = 0;
+	/*if (RUNNING_ON_VALGRIND) use_dlclose = 0;*/
 #endif
 
 	if (use_dlclose && p->lib) {	
@@ -108,17 +108,9 @@ int plugins_load(server *srv) {
 	for (i = 0; i < srv->srvconf.modules->used; i++) {
 		data_string *d = (data_string *)srv->srvconf.modules->data[i];
 		char *modules = d->value->ptr;
-		char *inst;
 	
-		buffer_reset(srv->tmp_buf);
-		/* for make distcheck */
-		if (NULL != (inst = strstr(LIBRARY_DIR, "/_inst/lib"))) {
-			buffer_copy_string_len(srv->tmp_buf, LIBRARY_DIR, strlen(LIBRARY_DIR) - strlen(inst));
-			buffer_append_string(srv->tmp_buf, "/_build/src/.libs");
-			buffer_append_string(srv->tmp_buf, inst + strlen("/_inst/lib") );
-		} else {
-			buffer_copy_string(srv->tmp_buf, LIBRARY_DIR);
-		}
+		buffer_copy_string_buffer(srv->tmp_buf, srv->srvconf.modules_dir);
+
 		buffer_append_string(srv->tmp_buf, "/");
 		buffer_append_string(srv->tmp_buf, modules);
 #if defined(__WIN32) || defined(__CYGWIN__)
@@ -245,13 +237,13 @@ int plugins_load(server *srv) {
 
 PLUGIN_TO_SLOT(PLUGIN_FUNC_HANDLE_URI_CLEAN, handle_uri_clean)
 PLUGIN_TO_SLOT(PLUGIN_FUNC_HANDLE_URI_RAW, handle_uri_raw)
-PLUGIN_TO_SLOT(PLUGIN_FUNC_HANDLE_PHYSICAL_PATH, handle_physical_path)
 PLUGIN_TO_SLOT(PLUGIN_FUNC_HANDLE_REQUEST_DONE, handle_request_done)
 PLUGIN_TO_SLOT(PLUGIN_FUNC_HANDLE_CONNECTION_CLOSE, handle_connection_close)
 PLUGIN_TO_SLOT(PLUGIN_FUNC_HANDLE_SUBREQUEST, handle_subrequest)
 PLUGIN_TO_SLOT(PLUGIN_FUNC_HANDLE_SUBREQUEST_START, handle_subrequest_start)
 PLUGIN_TO_SLOT(PLUGIN_FUNC_HANDLE_JOBLIST, handle_joblist)
 PLUGIN_TO_SLOT(PLUGIN_FUNC_HANDLE_DOCROOT, handle_docroot)
+PLUGIN_TO_SLOT(PLUGIN_FUNC_HANDLE_PHYSICAL, handle_physical)
 PLUGIN_TO_SLOT(PLUGIN_FUNC_CONNECTION_RESET, connection_reset)
 
 #undef PLUGIN_TO_SLOT
@@ -372,7 +364,6 @@ handler_t plugins_call_init(server *srv) {
 		
 		PLUGIN_TO_SLOT(PLUGIN_FUNC_HANDLE_URI_CLEAN, handle_uri_clean); 
 		PLUGIN_TO_SLOT(PLUGIN_FUNC_HANDLE_URI_RAW, handle_uri_raw); 
-		PLUGIN_TO_SLOT(PLUGIN_FUNC_HANDLE_PHYSICAL_PATH, handle_physical_path);
 		PLUGIN_TO_SLOT(PLUGIN_FUNC_HANDLE_REQUEST_DONE, handle_request_done);
 		PLUGIN_TO_SLOT(PLUGIN_FUNC_HANDLE_CONNECTION_CLOSE, handle_connection_close);
 		PLUGIN_TO_SLOT(PLUGIN_FUNC_HANDLE_TRIGGER, handle_trigger);
@@ -381,6 +372,7 @@ handler_t plugins_call_init(server *srv) {
 		PLUGIN_TO_SLOT(PLUGIN_FUNC_HANDLE_SUBREQUEST_START, handle_subrequest_start);
 		PLUGIN_TO_SLOT(PLUGIN_FUNC_HANDLE_JOBLIST, handle_joblist);
 		PLUGIN_TO_SLOT(PLUGIN_FUNC_HANDLE_DOCROOT, handle_docroot);
+		PLUGIN_TO_SLOT(PLUGIN_FUNC_HANDLE_PHYSICAL, handle_physical);
 		PLUGIN_TO_SLOT(PLUGIN_FUNC_CONNECTION_RESET, connection_reset);
 		PLUGIN_TO_SLOT(PLUGIN_FUNC_CLEANUP, cleanup);
 		PLUGIN_TO_SLOT(PLUGIN_FUNC_SET_DEFAULTS, set_defaults);
@@ -430,4 +422,5 @@ void plugins_free(server *srv) {
 	
 	free(srv->plugins.ptr);
 	srv->plugins.ptr = NULL;
+	srv->plugins.used = 0;
 }
