@@ -25,7 +25,7 @@
 #include "http_chunk.h"
 #include "fdevent.h"
 #include "connections.h"
-#include "file_cache.h"
+#include "stat_cache.h"
 #include "plugin.h"
 #include "joblist.h"
 
@@ -50,17 +50,15 @@
 #include <sys/resource.h>
 #endif
 
-const char *patches[] = { "SERVERsocket", "HTTPurl", "HTTPhost", "HTTPreferer", "HTTPuseragent", "HTTPcookie", NULL };
-
-
 #ifndef __sgi
 /* IRIX doesn't like the alarm based time() optimization */
 /* #define USE_ALARM */
 #endif
 
-static sig_atomic_t srv_shutdown = 0;
-static sig_atomic_t handle_sig_alarm = 1;
-static sig_atomic_t handle_sig_hup = 0;
+static volatile sig_atomic_t srv_shutdown = 0;
+static volatile sig_atomic_t gracefull_shutdown = 0;
+static volatile sig_atomic_t handle_sig_alarm = 1;
+static volatile sig_atomic_t handle_sig_hup = 0;
 
 #if defined(HAVE_SIGACTION) && defined(SA_SIGINFO)
 static void sigaction_handler(int sig, siginfo_t *si, void *context) {
@@ -69,6 +67,7 @@ static void sigaction_handler(int sig, siginfo_t *si, void *context) {
 
 	switch (sig) {
 	case SIGTERM: srv_shutdown = 1; break;
+	case SIGINT: gracefull_shutdown = 1; break;
 	case SIGALRM: handle_sig_alarm = 1; break;
 	case SIGHUP:  handle_sig_hup = 1; break;
 	case SIGCHLD: break;
@@ -78,6 +77,7 @@ static void sigaction_handler(int sig, siginfo_t *si, void *context) {
 static void signal_handler(int sig) {
 	switch (sig) {
 	case SIGTERM: srv_shutdown = 1; break;
+	case SIGINT: gracefull_shutdown = 1; break;
 	case SIGALRM: handle_sig_alarm = 1; break;
 	case SIGHUP:  handle_sig_hup = 1; break;
 	case SIGCHLD:  break;
@@ -125,11 +125,7 @@ static server *server_init(void) {
 	CLEAN(errorlog_buf);
 	CLEAN(response_range);
 	CLEAN(tmp_buf);
-	CLEAN(file_cache_etag);
-	CLEAN(range_buf);
-	CLEAN(empty_string);
-	
-	buffer_copy_string(srv->empty_string, "");
+	CLEAN(cond_check_buf);
 	
 	CLEAN(srvconf.errorlog_file);
 	CLEAN(srvconf.groupname);
@@ -165,10 +161,8 @@ static server *server_init(void) {
 	srv->fdwaitqueue = calloc(1, sizeof(*srv->fdwaitqueue));
 	assert(srv->fdwaitqueue);
 	
-	srv->file_cache = file_cache_init();
-	assert(srv->file_cache);
-	
 	srv->srvconf.modules = array_init();
+	srv->srvconf.modules_dir = buffer_init_string(LIBRARY_DIR);
 	
 	/* use syslog */
 	srv->errorlog_fd = -1;
@@ -176,14 +170,6 @@ static server *server_init(void) {
 
 	srv->split_vals = array_init();
 	
-	srv->config_patches = buffer_array_init();
-	for (i = 0; patches[i]; i++) {
-		buffer *b;
-		
-		b = buffer_array_append_get_buffer(srv->config_patches);
-		buffer_copy_string(b, patches[i]);
-	}
-	
 	return srv;
 }
 
@@ -194,8 +180,6 @@ static void server_free(server *srv) {
 		buffer_free(srv->mtime_cache[i].str);
 	}
 	
-	buffer_array_free(srv->config_patches);
-	
 #define CLEAN(x) \
 	buffer_free(srv->x);
 	
@@ -206,9 +190,7 @@ static void server_free(server *srv) {
 	CLEAN(errorlog_buf);
 	CLEAN(response_range);
 	CLEAN(tmp_buf);
-	CLEAN(file_cache_etag);
-	CLEAN(range_buf);
-	CLEAN(empty_string);
+	CLEAN(cond_check_buf);
 	
 	CLEAN(srvconf.errorlog_file);
 	CLEAN(srvconf.groupname);
@@ -217,6 +199,7 @@ static void server_free(server *srv) {
 	CLEAN(srvconf.bindhost);
 	CLEAN(srvconf.event_handler);
 	CLEAN(srvconf.pid_file);
+	CLEAN(srvconf.modules_dir);
 	
 	CLEAN(tmp_chunk_len);
 #undef CLEAN
@@ -226,7 +209,6 @@ static void server_free(server *srv) {
 #endif
 	fdevent_free(srv->ev);
 	
-	connections_free(srv);
 	free(srv->conns);
 	
 	if (srv->config_storage) {
@@ -242,9 +224,6 @@ static void server_free(server *srv) {
 			buffer_free(s->ssl_ca_file);
 			buffer_free(s->error_handler);
 			buffer_free(s->errorfile_prefix);
-			buffer_free(s->dirlist_css);
-			buffer_free(s->dirlist_encoding);
-			array_free(s->indexfiles);
 			array_free(s->mimetypes);
 			
 			free(s);
@@ -263,8 +242,10 @@ static void server_free(server *srv) {
 	joblist_free(srv, srv->joblist);
 	fdwaitqueue_free(srv, srv->fdwaitqueue);
 	
-	file_cache_free(srv, srv->file_cache);
-	
+	if (srv->stat_cache) {
+		stat_cache_free(srv->stat_cache);
+	}
+
 	array_free(srv->srvconf.modules);
 	array_free(srv->split_vals);
 	
@@ -290,18 +271,15 @@ static void show_help (void) {
 # define TEXT_SSL " (ssl)"
 #else
 # define TEXT_SSL
-#endif
-#ifdef HAVE_IPV6
-# define TEXT_IPV6 " -6         use IPv6\n"
-#else
-# define TEXT_IPV6
 #endif
 	char *b = PACKAGE_NAME "-" PACKAGE_VERSION TEXT_SSL " ("__DATE__ " " __TIME__ ")" \
 " - a light and fast webserver\n" \
 "usage:\n" \
 " -f <name>  filename of the config-file\n" \
+" -m <name>  module directory (default: "LIBRARY_DIR")\n" \
+" -p         print the parsed config-file in internal form, and exit\n" \
+" -t         test the config-file, and exit\n" \
 " -D         don't go to background (default: go to background)\n" \
-TEXT_IPV6 \
 " -v         show version\n" \
 " -h         show this help\n" \
 "\n"
@@ -313,6 +291,8 @@ TEXT_IPV6 \
 
 int main (int argc, char **argv) {
 	server *srv = NULL;
+	int print_config = 0;
+	int test_config = 0;
 	int i_am_root;
 	int o;
 	int num_childs = 0;
@@ -353,7 +333,7 @@ int main (int argc, char **argv) {
 #endif
 	srv->srvconf.dont_daemonize = 0;
 	
-	while(-1 != (o = getopt(argc, argv, "f:hvD"))) {
+	while(-1 != (o = getopt(argc, argv, "f:m:hvDpt"))) {
 		switch(o) {
 		case 'f': 
 			if (config_read(srv, optarg)) { 
@@ -361,6 +341,11 @@ int main (int argc, char **argv) {
 				return -1;
 			}
 			break;
+		case 'm':
+			buffer_copy_string(srv->srvconf.modules_dir, optarg);
+			break;
+		case 'p': print_config = 1; break;
+		case 't': test_config = 1; break;
 		case 'D': srv->srvconf.dont_daemonize = 1; break;
 		case 'v': show_version(); return 0;
 		case 'h': show_help(); return 0;
@@ -379,6 +364,26 @@ int main (int argc, char **argv) {
 		return -1;
 	}
 	
+	if (print_config) {
+		data_unset *dc = srv->config_context->data[0];
+		if (dc) {
+			dc->print(dc, 0);
+			fprintf(stderr, "\n");
+		} else {
+			/* shouldn't happend */
+			fprintf(stderr, "global config not found\n");
+		}
+	}
+
+	if (test_config) {
+		printf("Syntax OK\n");
+	}
+
+	if (test_config || print_config) {
+		server_free(srv);
+		return 0;
+	}
+	
 	/* close stdin and stdout, as they are not needed */
 	/* move stdin to /dev/null */
 	if (-1 != (fd = open("/dev/null", O_RDONLY))) {
@@ -462,7 +467,17 @@ int main (int argc, char **argv) {
 			}
 		}
 	}
-	
+
+	if (srv->event_handler == FDEVENT_HANDLER_SELECT) {
+		/* select limits itself
+		 *
+		 * as it is a hard limit and will lead to a segfault we add some safety
+		 * */
+		srv->max_fds = FD_SETSIZE - 200;
+	} else {
+		srv->max_fds = 4096;
+	}
+
 	if (i_am_root) {
 		struct group *grp = NULL;
 		struct passwd *pwd = NULL;
@@ -493,12 +508,23 @@ int main (int argc, char **argv) {
 				return -1;
 			}
 		}
-		
-		srv->max_fds = rlim.rlim_cur;
-#else
-		srv->max_fds = 4096;
+
+		if (srv->event_handler == FDEVENT_HANDLER_SELECT) {
+			srv->max_fds = rlim.rlim_cur < FD_SETSIZE - 200 ? rlim.rlim_cur : FD_SETSIZE - 200;
+		} else {
+			srv->max_fds = rlim.rlim_cur;
+		}
 #endif
-		
+		if (srv->event_handler == FDEVENT_HANDLER_SELECT) {
+			/* don't raise the limit above FD_SET_SIZE */
+			if (srv->max_fds > FD_SETSIZE - 200) {
+				log_error_write(srv, __FILE__, __LINE__, "sd", 
+						"can't raise max filedescriptors above",  FD_SETSIZE - 200,
+						"if event-handler is 'select'. Use 'poll' or something else or reduce server.max-fds.");
+				return -1;
+			}
+		}
+
 		if (NULL == (srv->ev = fdevent_init(srv->max_fds + 1, srv->event_handler))) {
 			log_error_write(srv, __FILE__, __LINE__,
 					"s", "fdevent_init failed");
@@ -566,6 +592,7 @@ int main (int argc, char **argv) {
 		if (srv->srvconf.username->used) setuid(pwd->pw_uid);
 #endif
 	} else {
+
 #ifdef HAVE_GETRLIMIT
 		if (0 != getrlimit(RLIMIT_NOFILE, &rlim)) {
 			log_error_write(srv, __FILE__, __LINE__,
@@ -573,12 +600,24 @@ int main (int argc, char **argv) {
 					strerror(errno));
 			return -1;
 		}
-		
-		srv->max_fds = rlim.rlim_cur;
-#else
-		srv->max_fds = 4096;
+
+		if (srv->event_handler == FDEVENT_HANDLER_SELECT) {
+			srv->max_fds = rlim.rlim_cur < FD_SETSIZE - 200 ? rlim.rlim_cur : FD_SETSIZE - 200;
+		} else {
+			srv->max_fds = rlim.rlim_cur;
+		}
 #endif
-		
+		if (srv->event_handler == FDEVENT_HANDLER_SELECT) {
+			/* don't raise the limit above FD_SET_SIZE */
+			if (srv->max_fds > FD_SETSIZE - 200) {
+				log_error_write(srv, __FILE__, __LINE__, "sd", 
+						"can't raise max filedescriptors above",  FD_SETSIZE - 200,
+						"if event-handler is 'select'. Use 'poll' or something else or reduce server.max-fds.");
+				return -1;
+			}
+		}
+
+	
 		if (NULL == (srv->ev = fdevent_init(srv->max_fds + 1, srv->event_handler))) {
 			log_error_write(srv, __FILE__, __LINE__,
 					"s", "fdevent_init failed");
@@ -592,6 +631,18 @@ int main (int argc, char **argv) {
 			return -1;
 		}
 	}
+
+	/* set max-conns */
+	if (srv->srvconf.max_conns > srv->max_fds) {
+		/* we can't have more connections than max-fds */
+		srv->max_conns = srv->max_fds;
+	} else if (srv->srvconf.max_conns) {
+		/* otherwise respect thw wishes of the user */
+		srv->max_conns = srv->srvconf.max_conns;
+	} else {
+		/* or use the default */
+		srv->max_conns = srv->max_fds;
+	}
 	
 	if (HANDLER_GO_ON != plugins_call_init(srv)) {
 		log_error_write(srv, __FILE__, __LINE__, "s", "Initialization of plugins failed. Going down.");
@@ -607,6 +658,9 @@ int main (int argc, char **argv) {
 	/* network is up, let's deamonize ourself */
 	if (srv->srvconf.dont_daemonize == 0) daemonize();
 #endif
+
+	srv->gid = getgid();
+	srv->uid = getuid();
 	
 	/* write pid file */
 	if (pid_fd != -1) {
@@ -631,6 +685,11 @@ int main (int argc, char **argv) {
 	for (i = 0; srv->config && i < srv->config->used; i++) {
 		data_unset *du = srv->config->data[i];
 		
+		/* all var.* is known as user defined variable */
+		if (strncmp(du->key->ptr, "var.", sizeof("var.") - 1) == 0) {
+			continue;
+		}
+
 		if (NULL == array_get_element(srv->config_touched, du->key->ptr)) {
 			log_error_write(srv, __FILE__, __LINE__, "sbs", 
 					"WARNING: unknown config-key:",
@@ -660,7 +719,11 @@ int main (int argc, char **argv) {
 		return -1;
 	}
 	
-	/* kqueue needs a reset AFTER daemonize() */
+	/* 
+	 * kqueue() is called here, select resets its internals, 
+	 * all server sockets get their handlers
+	 *
+	 * */
 	if (0 != network_register_fdevents(srv)) {
 		plugins_free(srv);
 		network_close(srv);
@@ -673,7 +736,6 @@ int main (int argc, char **argv) {
 	srv->cur_fds = open("/dev/null", O_RDONLY);
 	close(srv->cur_fds);
 	
-	
 #ifdef HAVE_SIGACTION
 	memset(&act, 0, sizeof(act));
 	act.sa_handler = SIG_IGN;
@@ -688,6 +750,7 @@ int main (int argc, char **argv) {
 	sigemptyset(&act.sa_mask);
 	act.sa_flags = 0;
 # endif
+	sigaction(SIGINT,  &act, NULL);
 	sigaction(SIGTERM, &act, NULL);
 	sigaction(SIGHUP,  &act, NULL);
 	sigaction(SIGALRM, &act, NULL);
@@ -701,6 +764,7 @@ int main (int argc, char **argv) {
 	signal(SIGTERM, signal_handler);
 	signal(SIGHUP,  signal_handler);
 	signal(SIGCHLD,  signal_handler);
+	signal(SIGINT,  signal_handler);
 #endif
 	
 #ifdef USE_ALARM
@@ -708,13 +772,31 @@ int main (int argc, char **argv) {
 	
 	/* setup periodic timer (1 second) */
 	if (setitimer(ITIMER_REAL, &interval, NULL)) {
-		log_error_write(srv, __FILE__, __LINE__, "setting timer failed");
+		log_error_write(srv, __FILE__, __LINE__, "s", "setting timer failed");
 		return -1;
 	}
 	
 	getitimer(ITIMER_REAL, &interval);
 #endif
 
+	/* might fail if user is using fam (not gamin) and famd isn't running */
+	if (NULL == (srv->stat_cache = stat_cache_init())) {
+#if defined(HAVE_FAM_H) && !defined(HAVE_FAMNOEXISTS)
+		log_error_write(srv, __FILE__, __LINE__, "s",
+			"FAMOpen2() failed.  Is famd running?");
+		return -1;
+#else
+		SEGFAULT();
+#endif
+	}
+
+#ifdef HAVE_FAM_H
+	/* setup FAM */
+	srv->stat_cache->fam_fcce_ndx = -1;
+	fdevent_register(srv->ev, FAMCONNECTION_GETFD(srv->stat_cache->fam), stat_cache_handle_fdevent, NULL);
+	fdevent_event_add(srv->ev, &(srv->stat_cache->fam_fcce_ndx), FAMCONNECTION_GETFD(srv->stat_cache->fam), FDEVENT_IN);
+#endif
+
 #ifdef HAVE_FORK	
 	/* start watcher and workers */
 	num_childs = srv->srvconf.max_worker;
@@ -801,7 +883,9 @@ int main (int argc, char **argv) {
 				
 				/* trigger waitpid */
 				srv->cur_ts = min_ts;
-				
+			
+				/* cleanup stat-cache */	
+				stat_cache_trigger_cleanup(srv);
 				/**
 				 * check all connections for timeouts 
 				 * 
@@ -891,31 +975,63 @@ int main (int argc, char **argv) {
 			}
 		}
 
-		/* handle out of fd condition */
-		if (!srv->sockets_disabled &&
-		    srv->cur_fds + srv->want_fds > srv->max_fds * 0.9) {
+		if (srv->sockets_disabled) {
+			/* our server sockets are disabled, why ? */
+
+			if ((srv->cur_fds + srv->want_fds < srv->max_fds * 0.8) && /* we have enough unused fds */
+			    (srv->conns->used < srv->max_conns * 0.9) &&
+			    (0 == gracefull_shutdown)) {
+				for (i = 0; i < srv->srv_sockets.used; i++) {
+					server_socket *srv_socket = srv->srv_sockets.ptr[i];
+					fdevent_event_add(srv->ev, &(srv_socket->fde_ndx), srv_socket->fd, FDEVENT_IN);
+				}
 			
-			/* disable server-fds */
+				log_error_write(srv, __FILE__, __LINE__, "s", "[note] sockets enabled again");
 			
-			for (i = 0; i < srv->srv_sockets.used; i++) {
-				server_socket *srv_socket = srv->srv_sockets.ptr[i];
-				fdevent_event_del(srv->ev, &(srv_socket->fde_ndx), srv_socket->fd);
+				srv->sockets_disabled = 0;
 			}
+		} else {
+			if ((srv->cur_fds + srv->want_fds > srv->max_fds * 0.9) || /* out of fds */
+			    (srv->conns->used > srv->max_conns) || /* out of connections */
+			    (gracefull_shutdown)) { /* gracefull_shutdown */ 
+
+				/* disable server-fds */
 			
-			log_error_write(srv, __FILE__, __LINE__, "s", "[note] sockets disabled, out-of-fds");
+				for (i = 0; i < srv->srv_sockets.used; i++) {
+					server_socket *srv_socket = srv->srv_sockets.ptr[i];
+					fdevent_event_del(srv->ev, &(srv_socket->fde_ndx), srv_socket->fd);
+
+					if (gracefull_shutdown) {
+						/* we don't want this socket anymore,
+						 *
+						 * closing it right away will make it possible for
+						 * the next lighttpd to take over (gracefull restart)
+						 *  */
+
+						fdevent_unregister(srv->ev, srv_socket->fd);
+						close(srv_socket->fd);
+						srv_socket->fd = -1;
+
+						/* network_close() will cleanup after us */
+					}
+				}
+		
+				if (gracefull_shutdown) {
+					log_error_write(srv, __FILE__, __LINE__, "s", "[note] gracefull shutdown started");
+				} else if (srv->conns->used > srv->max_conns) {
+					log_error_write(srv, __FILE__, __LINE__, "s", "[note] sockets disabled, connection limit reached");
+				} else {
+					log_error_write(srv, __FILE__, __LINE__, "s", "[note] sockets disabled, out-of-fds");
+				}
 			
-			srv->sockets_disabled = 1;
-		} else if (srv->sockets_disabled &&
-			   srv->cur_fds + srv->want_fds < srv->max_fds * 0.8) {
-			
-			for (i = 0; i < srv->srv_sockets.used; i++) {
-				server_socket *srv_socket = srv->srv_sockets.ptr[i];
-				fdevent_event_add(srv->ev, &(srv_socket->fde_ndx), srv_socket->fd, FDEVENT_IN);
+				srv->sockets_disabled = 1;
 			}
-			
-			log_error_write(srv, __FILE__, __LINE__, "s", "[note] sockets enabled, out-of-fds");
-			
-			srv->sockets_disabled = 0;
+		}
+
+		if (gracefull_shutdown && srv->conns->used == 0) {
+			/* we are in gracefull shutdown phase and all connections are closed
+			 * we are ready to terminate without harming anyone */
+			srv_shutdown = 1;
 		}
 		
 		/* we still have some fds to share */
@@ -1016,6 +1132,7 @@ int main (int argc, char **argv) {
 	/* clean-up */
 	log_error_close(srv);
 	network_close(srv);
+	connections_free(srv);
 	plugins_free(srv);
 	server_free(srv);
 	
