@@ -4,39 +4,52 @@ package LightyTest;
 use strict;
 use IO::Socket;
 use Test::More;
+use Socket;
+use Cwd 'abs_path';
 
 sub new {
 	my $class = shift;
 	my $self = {};
+	my $lpath;
 
 	$self->{CONFIGFILE} = 'lighttpd.conf';
-	$self->{BASEDIR} = (defined $ENV{'top_builddir'} ? $ENV{'top_builddir'} : '..');
-	$self->{SRCDIR}  = (defined $ENV{'srcdir'} ? $ENV{'srcdir'} : '.');
+
+	$lpath = (defined $ENV{'top_builddir'} ? $ENV{'top_builddir'} : '..');
+	$self->{BASEDIR} = abs_path($lpath);
+
+	$lpath = (defined $ENV{'top_builddir'} ? $ENV{'top_builddir'}."/tests/" : '.');
+	$self->{TESTDIR} = abs_path($lpath);
+
+	$lpath = (defined $ENV{'srcdir'} ? $ENV{'srcdir'} : '.');
+	$self->{SRCDIR} = abs_path($lpath);
+
 
 	$self->{LIGHTTPD_PATH} = $self->{BASEDIR}.'/src/lighttpd';
-	$self->{LIGHTTPD_PIDFILE} = '/tmp/lighttpd/lighttpd.pid';
-	$self->{PIDOF_PIDFILE} = '/tmp/lighttpd/pidof.pid';
+	$self->{LIGHTTPD_PIDFILE} = $self->{TESTDIR}.'/tmp/lighttpd/lighttpd.pid';
+	$self->{PIDOF_PIDFILE} = $self->{TESTDIR}.'/tmp/lighttpd/pidof.pid';
 	$self->{PORT} = 2048;
-	
+
+	my ($name, $aliases, $addrtype, $net) = gethostbyaddr(inet_aton("127.0.0.1"), AF_INET);
+
+	$self->{HOSTNAME} = $name;
+
 	bless($self, $class);
 
 	return $self;
 }
 
-sub pidof {
+sub listening_on {
 	my $self = shift;
-	my $prog = shift;
+	my $port = shift;
 
-	open F, "ps ax  | grep $prog | grep -v grep | awk '{ print \$1 }'|" or
-	open F, "ps -ef | grep $prog | grep -v grep | awk '{ print \$2 }'|" or
-	  return -1;
+	my $remote = 
+ 	  IO::Socket::INET->new(Proto    => "tcp",
+				PeerAddr => "127.0.0.1",
+				PeerPort => $port) or return 0;
 
-	my $pid = <F>;
-	close F;
+	close $remote;
 
-	if (defined $pid) { return $pid; }
-
-	return -1;
+	return 1;
 }
 
 sub stop_proc {
@@ -63,21 +76,22 @@ sub start_proc {
 	# pre-process configfile if necessary
 	#
 
-	my $pwd = `pwd`;
-	chomp($pwd);
-	unlink("/tmp/cfg.file");
-	system("cat ".$self->{SRCDIR}."/".$self->{CONFIGFILE}.' | perl -pe "s#\@SRCDIR\@#'.$pwd.'/'.$self->{BASEDIR}.'/tests/#" > /tmp/cfg.file');
+	unlink($self->{TESTDIR}."/tmp/cfg.file");
+	system("cat ".$self->{SRCDIR}."/".$self->{CONFIGFILE}.' | perl -pe "s#\@SRCDIR\@#'.$self->{BASEDIR}.'/tests/#" > '.$self->{TESTDIR}.'/tmp/cfg.file');
 
 	unlink($self->{LIGHTTPD_PIDFILE});
-	system($self->{LIGHTTPD_PATH}." -f /tmp/cfg.file");
-	# system("valgrind --tool=memcheck --show-reachable=yes --leak-check=yes --logfile=foo ".$lighttpd_path." -D -f /tmp/cfg.file &");
-	#
-	
+	if (1) {
+		system($self->{LIGHTTPD_PATH}." -f ".$self->{TESTDIR}."/tmp/cfg.file -m ".$self->{BASEDIR}."/src/.libs");
+	} else {
+		system("valgrind --tool=memcheck --show-reachable=yes --leak-check=yes --logfile=foo ".$self->{LIGHTTPD_PATH}." -D -f ".$self->{TESTDIR}."/tmp/cfg.file -m ".$self->{BASEDIR}."/src/.libs &");
+	}
+
 	select(undef, undef, undef, 0.1);
+	if (not -e $self->{LIGHTTPD_PIDFILE} or 0 == kill 0, `cat $self->{LIGHTTPD_PIDFILE}`) {
+		select(undef, undef, undef, 2);	
+	}
 
-	# sleep(1);
-
-	unlink("/tmp/cfg.file");
+	unlink($self->{TESTDIR}."/tmp/cfg.file");
 
 	# no pidfile, we failed
 	if (not -e $self->{LIGHTTPD_PIDFILE}) {
@@ -107,8 +121,12 @@ sub handle_http {
 	my $remote = 
  	  IO::Socket::INET->new(Proto    => "tcp",
 				PeerAddr => $host,
-				PeerPort => $self->{PORT})
-	  or return -1;
+				PeerPort => $self->{PORT});
+
+	if (not defined $remote) {
+		diag("connect failed: $!");
+	       	return -1;
+	}
 
 	$remote->autoflush(1);
 
@@ -130,7 +148,7 @@ sub handle_http {
 	close $remote;
 
 	my $href;
-	foreach $href (@response) {
+	foreach $href ( @{ $t->{RESPONSE} }) {
 		# first line is always response header
 		my %resp_hdr;
 		my $resp_body;
@@ -167,7 +185,13 @@ sub handle_http {
 
 		# check length
 		if (defined $resp_hdr{"content-length"}) {
-			($resp_body, $lines) = split("^.".$resp_hdr{"content-length"}, $lines, 2);
+			$resp_body = substr($lines, 0, $resp_hdr{"content-length"});
+			if (length($lines) < $resp_hdr{"content-length"}) {
+				$lines = "";
+			} else {
+				$lines = substr($lines, $resp_hdr{"content-length"});
+			}
+			undef $lines if (length($lines) == 0);
 		} else {
 			$resp_body = $lines;
 			undef $lines;
@@ -188,6 +212,7 @@ sub handle_http {
 		}
 
 		if (defined $href->{'HTTP-Content'}) {
+			$resp_body = "" unless defined $resp_body;
 			if ($href->{'HTTP-Content'} ne $resp_body) {
 				diag(sprintf("body failed: expected '%s', got '%s'\n", $href->{'HTTP-Content'}, $resp_body));
 				return -1;
